<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

if(!class_exists('User'))
	include('User.class.php');
	
if(!class_exists('Node'))
	include('Node.class.php');

class Admin extends User {
	private $DB = null;
	
	private $node = array();
	private $nodeLimits = array();
	
	public function __construct($admin)
	{
		$this->DB =& OpenVCP::call('DB', 'getReference');
		
    	if(is_numeric($admin) && $admin > 0)
    	{
    		$admin = $this->DB->select($this->DB->table_admin, NULL, NULL, array('id' => $admin));
    	}
    	else if(is_string($admin) && $admin != "")
    	{
    		$admin = $this->DB->select($this->DB->table_admin, NULL, NULL, array('loginname' => $admin));
    	}
    	else
    	{
    		OpenVCP::coreError("admin does not exist");
    	}
		parent::__construct($admin[0]);
	}

	/**
	 * returns an array with the images of the admin
	 * 
	 * @return array $images
	 */
	public function getImages()
	{
		$imageids = $this->DB->select($this->DB->table_admin_images, NULL, NULL, array('adminid' => parent::$this->userinfo['id']));
		$images = null;
		if(is_array($imageids))
			foreach($imageids as $image)
			{
				$imagename = $this->DB->select($this->DB->table_images, NULL, array('id', 'name', 'alias'), array('id' => $image['imageid']));
				if($imagename[0]['alias'] != "")
					$images[$imagename[0]['alias']] = $imagename[0]['name'];
				else $images[] = $imagename[0]['name'];
			}
		return $images;
	}
	
	public function getImagesExtended()
	{
		$imageids = $this->DB->select($this->DB->table_admin_images, NULL, NULL, array('adminid' => $this->id));
		$images = null;
		foreach($imageids as $image)
		{
			$imagename = $this->DB->select($this->DB->table_images, NULL, array('id', 'name', 'alias'), array('id' => $image['imageid']));
			$images[$imagename[0]['name']]['name'] = $imagename[0]['name'];
			$images[$imagename[0]['name']]['id'] = $imagename[0]['id'];
			$images[$imagename[0]['name']]['alias'] = $imagename[0]['alias'];
		}
		return $images;		
	}
	
	public function getNodes()
	{
		return $this->DB->joinselect(array($this->DB->table_nodes, $this->DB->table_admin_nodes), NULL, array($this->DB->table_nodes => 0), array(array( $this->DB->table_admin_nodes => 'nodeid', $this->DB->table_nodes => 'id'), 'AND', array($this->DB->table_admin_nodes => 'adminid', parent::$this->userinfo['id'])), array('name'=>'ASC'));
	}
	
	public function getVserverCount($nodeid)
	{
		$vservers = $this->DB->select($this->DB->table_vservers, NULL, NULL, array('adminid' => parent::$this->userinfo['id'], 'AND', 'nodeid' => $nodeid));
		return count($vservers);
	}

	/**
	 * get's the admin's settings or return the global setting
	 * 
	 * @param string $key
	 * @return string setting
	 */
	public function getSetting($key)
	{
		$settings = $this->DB->select($this->DB->table_admin_settings, NULL, NULL, array('adminid' => $this->userinfo['id'], 'AND', 'key' => $key));
		if(is_array($settings[0]) && $settings[0]['value'] != "")
		{
			return $settings[0]['value'];
		}
		else
		{
			$settings = $this->DB->select($this->DB->table_settings, NULL, NULL, array('key' => $key));
			return (is_array($settings) && $settings[0]['value'] != "") ? $settings[0]['value'] : false;
		}
	}
	
	private function _loadNode($nodeid)
	{
		$this->node[$nodeid] = new Node($nodeid);
		return true;
	}
	
	private function _loadNodeLimits($nodeid) {
		$limits = $this->DB->select($this->DB->table_admin_nodes, NULL, NULL, array('adminid' => parent::$this->userinfo['id'], 'AND', 'nodeid' => $nodeid));
		$this->nodeLimits[$nodeid] = $limits[0];
		return true;
	}

	public function getLimit($nodeid, $limit, $type = "")
	{
		if($this->node[$nodeid] == null)
			$this->_loadNode($nodeid);
		if(!is_array($this->nodeLimits[$nodeid]))
			$this->_loadNodeLimits($nodeid);
		return $this->nodeLimits[$nodeid][($type != "" ? $limit."_".$type : $limit)."_limit"];
	}

	public function setLimit($nodeid, $limit, $type = "", $value)
	{
		if($this->node[$nodeid] == null)
			$this->_loadNode($nodeid);
		if(!is_array($this->nodeLimits[$nodeid]))
			$this->_loadNodeLimits($nodeid);
		$this->nodeLimits[$nodeid][($type != "" ? $limit."_".$type : $limit)."_limit"] = $value;
		$this->DB->update($this->DB->table_admin_nodes, array(($type != "" ? $limit."_".$type : $limit)."_limit" => $value), array('nodeid' => $nodeid, 'AND', 'adminid' => parent::$this->userinfo['id']));
		return $this->nodeLimits[$nodeid][($type != "" ? $limit."_".$type : $limit)."_limit"];
	}
	
	public function createURL()
	{
		return "super/admins/admin/".parent::$this->userinfo['loginname'];
	}
	
	/**
	 * return a list of all admins
	 */
	public static function getAdminList()
	{
		$DB =& OpenVCP::call('DB', 'getReference');
		return $DB->select($DB->table_admin, NULL, array('id', 'loginname'));
	}
	
}
?>
