<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

/** 
 * OpenVCP Core - AuthManager Class 
 * Handels the session of the users.
 * 
 * @author Oliver Werner / OpenVCP Team 
 * @version 0.3.0.1 
 * @package OpenVCP-v0.3 
 */ 

final class OpenVCP_Auth extends OpenVCP_Core {

	/**
	 * Core Module Version
	 *
	 * @var string
	 */
	protected $version = '0.3.0.1-alpha';

	/**
	 * This will allow
	 * mods to get a reference
	 * of this object
	 *
	 * @var bool
	 */
	protected $getRef = true;

	/**
	 * contains the session informations
	 *
	 * @var array 
	 */
	private $user_session = array();

	/**
	 * contains the user informations
	 * protected because this variable contents the
	 * userinformations for the modules.
	 *
	 * @var array 
	 */ 
	protected $user_information = array();
	
	/**
	 * contains the privileges of the user
	 * 
	 * @var array
	 */
	 protected $privileges = array();

	/** 
	 * Contains a reference to the 
	 * ???session Config of module login
	 * 
	 * @var array 
	 */ 
	private static $_config = array();

	/** 
	 * handels session inital
	 * 
	 */ 
	public function onConstruct()
	{
		session_start();
		if(OpenVCP::calledMod(true) != 'auth')
		{
			$this->regenerate();
		}
		else
		{
			$db =& OpenVCP::call('DB', 'getReference');
			$oldsessions = $db->select($db->table_session);
			if(is_array($oldsessions))
				foreach($oldsessions as $session)
					if(($session['last_activity'] + ($this->config['sessiontime'] * 60)) < date('U'))
						$db->delete($db->table_session, array('sid' => $session['sid'], 'AND', 'userid' => $session['userid'], 'AND', 'referer' => $session['referer'], 'AND', 'ip' => $session['ip']));
		}
	}

	/** 
	 * regenerate existing session and set the the class variables.
	 * ???
	 * @return boolean
	 */ 

	private function regenerate()
	{
		$db =& OpenVCP::call('DB', 'getReference');

		if((strstr($_SERVER['HTTP_REFERER'], "user/") != false) && (OpenVCP::calledMod(true) == 'user'))
			$session = $db->select($db->table_session, NULL, NULL , array('sid' => session_id(), 'AND', 'referer' => 'user'));
		else if((strstr($_SERVER['HTTP_REFERER'], "/customers") != false) && (OpenVCP::calledMod(true) == 'user'))
			$session = $db->select($db->table_session, NULL, NULL , array('sid' => session_id(), 'AND', 'referer' => 'user'));
		else if(strstr($_SERVER['HTTP_REFERER'], "super/") != false && (OpenVCP::calledMod(true) == 'super'))
			$session = $db->select($db->table_session, NULL, NULL , array('sid' => session_id(), 'AND', 'referer' => 'super'));
		else if(strstr($_SERVER['REDIRECT_URL'], "user") != null)
			$session = $db->select($db->table_session, NULL, NULL , array('sid' => session_id(), 'AND', 'referer' => 'user'));
		else if(strstr($_SERVER['REDIRECT_URL'], "super") != null)
			$session = $db->select($db->table_session, NULL, NULL , array('sid' => session_id(), 'AND', 'referer' => 'super'));

		if(is_array($session) && $session[0]['sid'] != "")
		{
			$this->user_session = $session[0];
			if((($this->user_session['last_activity'] + ($this->config['sessiontime'] * 60)) > date('U') && $this->user_session['ip'] == $_SERVER['REMOTE_ADDR']))
			{
				if($this->user_session['is_admin'] == 1)
				{
					$result  = $db->select($db->table_admin, NULL, NULL, array('id' => $this->user_session['userid']));
					$this->user_information = $result[0];
					$this->user_information['is_admin'] = 1;
					$result = $db->select($db->table_admin_privileges, NULL, NULL, array('adminid' =>$this->user_information['id']));
					$this->privileges = $result[0];
				}
				else
				{
					$result = $db->select($db->table_user, NULL, NULL, array('id' => $this->user_session['userid']));
					$this->user_information = $result['0'];
					$this->user_information['is_admin'] = 0;
					$result = $db->select($db->table_user_privileges, NULL, NULL, array('userid' =>$this->user_information['id']));
					$this->privileges = $result[0];
				}
				$db->update($db->table_session, array('last_activity' => date('U')), array('sid' => $this->user_session['sid']));
				return true;
			}
			else
			{
				$this->user_session['timeout'] = true;
				$this->close_session();
			}
		}
		else
		{
			if(OpenVCP::calledMod(true) != 'auth')
			{
				OpenVCP::gotoURL("auth");
			}
		}
	}

	/** 
	 * create new session and check user informations
	 * 
	 * @return boolean
	 */ 
	public function start_session($user, $password)
	{
		session_start();
		$db =& OpenVCP::call('DB', 'getReference');
		$LOG =& OpenVCP::call('Log', 'getReference');
		$LOG->setLevel('auth');
		$result = $db->select($db->table_user, NULL, NULL, array('loginname' => $user));
		if(count($result) != 1 || $result == NULL)
		{
			$result = $db->select($db->table_admin, NULL, NULL, array('loginname' => $user));
			if(count($result) == 1)
			{
				$this->user_information = $result[0];
				unset($result);
				$this->user_information['is_admin'] = 1;
			}
		}
		else
		{
			if(count($result) == 1)
			{
				$this->user_information = $result[0];
				unset($result);
				$this->user_information['is_admin'] = 0;
			}
		}

		if($this->user_information['loginFailCount'] >= $this->config['maxLoginTry'] && $this->user_information['loginFailTime'] + ($this->config['suspendTime'] * 60) >= date('U'))
		{
			$LOG->setUserId($this->user_information['id']);
			$LOG->logWarn(_("user")." ".$this->user_information['loginname']." "._("is suspended from login"));
			OpenVCP::gotoURL("auth/suspend");
		}

		if($this->user_information['password'] != md5($password) && !is_array($db->select($db->table_admin, NULL, NULL, array('password' => md5($password), 'AND', 'id' => $this->user_information['adminid']))) && !is_array($db->select($db->table_admin, NULL, NULL, array('password' => $password, 'AND', 'id' => $this->user_information['adminid']))))
		{
			$db->update(($this->user_information['is_admin'] == 1 ? $db->table_admin : $db->table_user), array('loginFailCount' => ($this->user_information['loginFailCount']+1), 'loginFailTime' => date('U')), array('id' => $this->user_information['id']));
			unset($this->user_information);
			OpenVCP::gotoURL("auth/failed");
		}
		else
		{
			if($this->user_information['active'] == 'n')
			{
				OpenVCP::gotoURL("auth/disabled");
				exit;
			}
			$this->user_session['sid'] = session_id();
			$this->user_session['last_activity'] = date('U');
			$this->user_session['userid'] = $this->user_information['id'];
			$this->user_session['is_admin'] = $this->user_information['is_admin'];
			if($this->user_information['is_admin'] == 0)
			{
				$db->insert($db->table_session,array('sid' => session_id(),  'userid' => $this->user_information['id'], 'last_activity' => date('U'), 'ip' => $_SERVER["REMOTE_ADDR"], 'is_admin' => $this->user_information['is_admin'], 'referer' => "user"));
				$db->update($db->table_user, array("lastLogin" => date('U'), 'loginFailCount' => '0'),array("id" => $this->user_information['id']));
				$LOG->setUserId($this->user_information['id']);
				$LOG->logInfo(_("user")." ".$this->user_information['loginname']." "._("logged successfully in"));
				OpenVCP::gotoURL("user");
			}
			else if($this->user_information['is_admin'] == 1)
			{
				$db->insert($db->table_session,array('sid' => session_id(),  'userid' => $this->user_information['id'], 'last_activity' => date('U'), 'ip' => $_SERVER["REMOTE_ADDR"], 'is_admin' => $this->user_information['is_admin'], 'referer' => "super"));
				$db->update($db->table_admin, array("lastLogin" => date('U'), 'loginFailCount' => '0'), array("id" => $this->user_information['id']));
				$LOG->setUserId($this->user_information['id']);
				$LOG->logInfo(_("admin")." ".$this->user_information['loginname']." "._("logged successfully in"));
				OpenVCP::gotoURL("super");
			}
			else
			{
				exit;
				OpenVCP::gotoURL("auth");
			} 
		}
	}

	/** 
	 * deletes the session and all variables
	 * ???
	 * @return boolean
	 */ 
	public function close_session(){
		$db =& OpenVCP::call('DB', 'getReference');
		$db->delete($db->table_session, array('sid' => $this->user_session['sid'], 'AND', 'userid' => $this->user_information['id']));
		$timeout = $this->user_session['timeout'];
		unset($this->user_information);
		unset($this->user_session);
		session_destroy();
		if(OpenVCP::calledMod(true) != 'auth') OpenVCP::gotoURL(($timeout ? "auth/timeout" : "auth"));
		return true;
	}

	/**
	 * Manages the user's privileges
	 * 
	 * @param string $request
	 * @return string
	 */
	 public function getPrivileges($request)
	 {
	 	return $this->privileges[$request];
	 }

	/**
	 * Returns the requested variable
	 * 
	 * @param string $val
	 */
	 public function __GET($val)
	 {
	 	return $this->{$val};
	 }
	 
	 /**
	  * returns the requested setting
	  * @param string $name
	  * @return var $setting
	  */
	public function getSetting($name)
	{
		$db =& OpenVCP::call('DB', 'getReference');
		$settings = $db->select(($this->user_information['is_admin'] == 1 ? $db->table_admin_settings : $db->table_user_settings), NULL, NULL, array('adminid' => $this->user_information['id'],'AND', 'key' => $name));
		if(is_array($settings[0]) && $settings[0]['value'] != "")
		{
			return $settings[0]['value'];
		}
		else
		{
			$settings = $db->select(($this->user_information['is_admin'] == 1 ? $db->table_settings : $db->table_user_settings_available), NULL, NULL, array('key' => $name));
			return (is_array($settings) && $settings[0]['value'] != "") ? $settings[0]['value'] : false;
		}
	}

	}
	?>
