<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

/**
 * The DB extension for the OpenVCP Interface.
 * This extension provides methods to 
 * work with a MySQL database.
 * 
 * @author  Jean-Pascal Hoevel / Oliver Werner
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 * @example ../docs/db/example.php
 */
final class OpenVCP_DB extends OpenVCP_Core {
	
	/**
	 * Core Module Version
	 *
	 * @var string
	 */
	protected $version = '0.3.0.1-alpha';
	
	/**
	 * This will allow
	 * mods to get a reference
	 * of this object
	 *
	 * @var bool
	 */
	protected $getRef = true;
	
	/**
	 * Contains the MySQL DB
	 * resource
	 *
	 * @var resource
	 */
	protected $dbResource = null;
	
	/**
	 * Contains the defined tables
	 *
	 * @var array
	 */
	protected $tables = array();
	
	/**
	 * @internal 
	 */
	public function onConstruct(){
		foreach($this->config['table'] as $table){
			$table = explode(' ', $table);
			$this->tables[$table[0]] = $table[1];
		}
		$this->dbResource = @mysql_connect($this->config['host'], $this->config['user'], $this->config['passwd']);
		if(!$this->dbResource){
			OpenVCP::coreError(_("Failed to contact MySQL, returned error message: %1%"), 1, mysql_error());
		}
		else if(!mysql_select_db($this->config['db'])){
			OpenVCP::coreError(_("Failed to select the database, returned error message: %1%"), 1, $this->config['db'], mysql_error());
		}
	}
	
	/**
	 * @internal 
	 */
	public function onDestruct(){
		if(is_resource($this->dbResource)) @mysql_close($this->dbResource);
	}
	
	/**
	 * Getter for table defines
	 *
	 * @param string $table TABLE_NAME
	 */
	public function __get($table){
		return $this->tables[strtoupper($table)];
	}
	
	/**
	 * Inserts data into the DB.
	 * 
	 * The syntax of this function is
	 * "multirow" desiged, so your
	 * $dataSet array should look some
	 * like this:<br/>
	 * $dataSet[0] = array("A Name", "A Email")<br/>
	 * $dataSet[1] = array("A Name", null)<br/>
	 * $dataSet[2] = array(null, null)<br/>
	 * and so on..
	 *
	 * @param array  $dataSet  A set of data to insert.
	 * @param array  $fieldSet A set of fields to insert.
	 * @param string $table Name of an another table then the current
	 * @return bool
	 */
	public function insert($table, array $dataSet, array $fieldSet = null){
		if(is_resource($this->dbResource)){
			$command  = "INSERT INTO " . $this->config['prefix'] . $table;
			$command .= is_array($fieldSet) ? "(`" . implode('`,`', $fieldSet) . "`) VALUES " : " VALUES ";
			if(is_array($dataSet[0])){
				foreach($dataSet as $data){
					$command .= "(";
					foreach($data as $value){
						$command .= "'$value'" . (end($data) === $value ? '' : ',');
					}
					$command .= ")" . (end($dataSet) === $data ? '' : ',');
				}
			}
			else if(is_array($dataSet)){
				$command .= "('" . implode("','", $dataSet) . "')";
			}
			#echo $command;
			if(!@mysql_query($command)){
				OpenVCP::coreError("MySQL query failed, returned error message: %1%", 1, mysql_error());
				return false;
			}
			else return mysql_insert_id();
		}
		else {
			OpenVCP::coreError("MySQL query failed, no connection opened.", 1);
			return false;
		}
	}

	/**
	 * Updates data in a table
	 * 
	 * The syntax of this function is
	 * similar to the others, except
	 * that you only can do one update per
	 * call.
	 *
	 * @param array  $dataSet A set of data to update.
	 * @param array  $compare 
	 * @param string $table
	 * @return bool
	 */
	public function update($table, array $dataSet, array $compare = null){
		if(is_resource($this->dbResource)){
			$i = 1;
			$c = "WHERE";
			foreach($dataSet as $field => $value){
				$u .= " `$field` = '$value' ,";
			}
			$u = rtrim($u, ",");
			if(is_array($compare)){
				foreach($compare as $field => $value){
					if($i % 2 === 0){
						$value = strtoupper($value);
						if($value === 'OR' || $value === 'AND'){
							$c .= " $value ";
						}
						else OpenVCP::coreError("Every second array value must be AND or OR");
					}
					else $c .= " `$field` = '$value' ";
					$i++;
				}
			}
			#echo "UPDATE `" . $this->config['prefix'] . $table . "` SET $u $c";
			if(!@mysql_query("UPDATE `" . $this->config['prefix'] . $table . "` SET $u $c")){
				OpenVCP::coreError("MySQL query failed, returned error message: %1%", 1, mysql_error());
				return false;
			}
			else return true;
		}
		else {
			OpenVCP::coreError("MySQL query failed, no connection opened.", 1);
			return false;
		}
	}

	/**
	 * Deletes data from a table.
	 *
	 * @param string $table
	 * @param array  $deleteSet
	 * @return bool
	 */
	public function delete($table, array $deleteSet){
		if(is_resource($this->dbResource)){
			$i = 1;
			$c = "WHERE";
			if(is_array($deleteSet[0])){
				foreach($deleteSet as $comp){
					foreach($comp as $field => $value){
						if($i % 2 === 0){
							$value = strtoupper($value);
							if($value === 'OR' || $value === 'AND'){
								$c .= " $value ";
							}
							else {
								OpenVCP::coreError("Every second array value must be AND or OR", 1);
								return false;
							}
						}
						else $c .= " `$field` LIKE '$value' ";
						$i++;
					}
					$i = 1;
					if(!@mysql_query("DELETE FROM `" . $this->config['prefix'] . $table . "` $c LIMIT 1")){
						OpenVCP::coreError("MySQL query failed, returned error message: %1%", 1, mysql_error());
						$wasError = true;
					}
					$c = "WHERE";
				}
			}
			else if(is_array($deleteSet)){
				foreach($deleteSet as $field => $value){
					if($i % 2 === 0){
						$value = strtoupper($value);
						if($value === 'OR' || $value === 'AND'){
							$c .= " $value ";
						}
						else OpenVCP::coreError("Every second array value must be AND or OR");
					}
					else $c .= " `$field` LIKE '$value' ";
					$i++;
				}
				if(!@mysql_query("DELETE FROM `" . $this->config['prefix'] . $table . "` $c LIMIT 1")){
					OpenVCP::coreError("MySQL query failed, returned error message: %1%", mysql_error());
					$wasError = true;
				}
			}
			return isset($wasError) ? false : true;
		}
		else {
			OpenVCP::coreError("MySQL query failed, no connection opened.", 1);
			return false;
		}
	}
	
	/**
	 * Truncates a table
	 *
	 * @param  string $table
	 * @return bool
	 */
	public function truncate($table){
		if(is_resource($this->dbResource)){
			if(!@mysql_query("TRUNCATE TABLE `" . $this->config['prefix'] . $table)){
				OpenVCP::coreError("MySQL query failed, returned error message: %1%", 1, mysql_error());
				return false;
			}
			else return true;
		}
		else {
			OpenVCP::coreError("MySQL query failed, no connection opened.", 1);
			return false;
		}
	}

	/**
	 * Selects data from the DB an return 
	 * the result as assoc array
	 *
	 * @param string $table
	 * @param int    $limits
	 * @param array  $fields
	 * @param array  $compare
	 * @param array  $order
	 * @return bool
	 */
	public function select($table, array $limits = null, array $fields = null, array $compare = null, array $order = null){
		if(is_resource($this->dbResource)){
			$t = $this->config['prefix'] . $table;
			$f = is_array($fields) ? "`" . implode("`,`", $fields) . "`" : "*";
			$l = is_array($limits) ? "LIMIT " . $limits[0] . (!is_int($limits[1]) ? "" : "," . $limits[1]) : "";
			if(is_array($compare)){
				$i = 1;
				$c = "WHERE ";
				foreach($compare as $field => $value){
					if($i % 2 === 0){
						$value = strtoupper($value);
						if($value === 'OR' || $value === 'AND'){
							$c .= " $value ";
						}
						else {
							OpenVCP::coreError("Every second array value must be AND or OR", 1);
							return false;
						}
					}
					else $c .= " `$field` LIKE '$value' ";
					$i++;
				}
			}
			if(is_array($order) && count($order) == 1)
			{
				$o = "ORDER BY ";
				foreach($order as $key => $value)
				{
					 $o .= "`". $key . "` " . $value;
				}
				
			}
			#echo "SELECT $f FROM $t $c $o $l\n";
			if(!$result = @mysql_query("SELECT $f FROM $t $c $o $l")){
				OpenVCP::coreError("MySQL query failed, returned error message: %1%", 1, mysql_error());
				return false;
			}
			else {
				while(($r = mysql_fetch_assoc($result)) !== false){
					$return[] = $r;
				}
				return !is_array($return) ? false : $return;
			}
		}
		else {
			OpenVCP::coreError("MySQL query failed, no connection opened.", 1);
			return false;
		}
	}

	/**
	 * Selects data from the DB an return 
	 * the result as assoc array
	 *
	 * @param array $tables
	 * @param int    $limits
	 * @param array  $fields
	 * @param array  $compare
	 * @param array  $order
	 * @return bool
	 */
	public function joinselect(array $tableArray = null, array $limits = null, array $fields = null, array $compare = null, array $order = null){
		if(is_resource($this->dbResource)){
			$t = "";
			foreach($tableArray as $key => $table)
			{
				$t .= $this->config['prefix'] . $table . ", ";
			}
			$t = rtrim($t, ', ');
			$f = "";
			foreach($fields as $table => $field)
			{
				if(is_array($field))
					foreach($field as $subfield)
						$f .= "".$this->config['prefix'] . $table . "." . $subfield . ", ";
				else if(is_string($field))
					$f .= "".$this->config['prefix'] . $table . "." . $field . ", ";
				else
					$f .= "".$this->config['prefix'] . $table . ".*, ";
			}
			$f = rtrim($f, ', ');
			$l = is_array($limits) ? "LIMIT " . $limits[0] . (!is_int($limits[1]) ? "" : "," . $limits[1]) : "";
			if(is_array($compare)){
				$i = 1;
				$c = "WHERE ";
				foreach($compare as $key => $value){
					if($i % 2 === 0){
						$value = strtoupper($value);
						if($value === 'OR' || $value === 'AND'){
							$c .= " $value ";
						}
						else {
							OpenVCP::coreError("Every second array value must be AND or OR", 1);
							return false;
						}
					}
					else
					{
						if(is_array($value))
							foreach($value as $table => $field)
								if(!is_numeric($table))
									$c .= $this->config['prefix'] . $table . "." . $field . " = ";
								else
									$c .= $field . " = ";
						$c = rtrim($c, ' = ');
					} 
					$i++;
				}
			}
			if(is_array($order) && count($order) == 1)
			{
				$o = "ORDER BY ";
				foreach($order as $key => $value)
				{
					 $o .= "`". $key . "` " . $value;
				}
				
			}
			echo "SELECT $f FROM $t $c $o $l\n";
			if(!$result = @mysql_query("SELECT $f FROM $t $c $o $l")){
				OpenVCP::coreError("MySQL query failed, returned error message: %1%", 1, mysql_error());
				return false;
			}
			else {
				while(($r = mysql_fetch_assoc($result)) !== false){
					$return[] = $r;
				}
				return !is_array($return) ? false : $return;
			}
		}
		else {
			OpenVCP::coreError("MySQL query failed, no connection opened.", 1);
			return false;
		}
	}

	/**
	 * returns an asoziative array which shows the 
	 * SHOW COLUMNS FROM sometable
	 * 
	 * @param string $table
	 */
	public function returnColums($table)
	{
		$result = mysql_query("SHOW COLUMNS FROM ".$this->config['prefix'] . $table);
		if(!$result){
			#echo 'Abfrage konnte nicht ausgef?hrt werden: ' . mysql_error();
		   return false;
		}

		if (mysql_num_rows($result) > 0) 
		{
		   while ($row = mysql_fetch_assoc($result)) 
		   {
		       $colums[] = $row;
		   }
		}
		return $colums;
	}
}
?>
