<?php
/**
 * OpenVCP GUI Core Extension
 * This extension is a built in one,
 * so there no need to load it anywhere.
 * It provides methods to create
 * the output of the interface.
 *
 * @author  Jean-Pascal Hoevel >= 0.3.0.1
 * @author  Oliver Werner < 0.3.0.1
 * @version 0.3.0.5
 * @package OpenVCP-v0.3
 */
final class OpenVCP_GUI extends OpenVCP_Core {

	/**
	 * Core module version
	 *
	 * @var string
	 */
	protected $version = '0.3.0.5';

	/**
	 * Allow references
	 *
	 * @var bool
	 */
	protected $getRef = true;

	/**
	 * Enter description here...
	 *
	 * @var HTML_Template_IT
	 */
	protected $pearIT = null;

	/**
	 * Contains the template
	 * name
	 *
	 * @var string
	 */
	protected $templateName = "";

	/**
	 * Contains the template
	 * html data
	 *
	 * @var string
	 */
	protected $template = "";

	/**
	 * Contains the content
	 * html data.
	 *
	 * @var string
	 */
	protected $contentData = "";

	/**
	 * Contains the content
	 * menu html data.
	 *
	 * @var string
	 */
	protected $contentMenu = "";

	/**
	 * Contains additional content
	 * for the menu html data
	 *
	 * @var array
	 */
	protected $contentMenuAdd = array();

	/**
	 * Contains the header
	 * menu html data.
	 *
	 * @var string
	 */
	protected $headerMenu = "";

	/**
	 * Contains the page title.
	 *
	 * @var string
	 */
	protected $pageTitle = "";

	/**
	 * Contains elementid
	 *
	 * @var int
	 */
	protected $elementId = 800;

	/**
	 * Info variable for construct and destruct
	 *
	 * @var array
	 */
	protected $info = array();

	/**
	 * Contains whether init run or not
	 */
	protected $initRun = true;

	/**
	 * makes colored rows
	 */
	private $tableRowCounter = array();

	/**
	 * @internal
	 */
	public function onConstruct()
	{
		ob_start();
		$auth = & OpenVCP::call('Auth', 'getReference');
		$db =& OpenVCP::call('DB', 'getReference');
		if($auth->user_information['is_admin'])
		{
			$theme = $db->select($db->table_admin_settings, NULL, array('value'), array('adminid' => $auth->user_information['id'], 'AND', 'key' => 'theme'));
		}
		else
		{
			$theme = $db->select($db->table_user_settings, NULL, array('value'), array('userid' => $auth->user_information['id'], 'AND', 'key' => 'theme'));
		}
		if($theme != null)
		{
			$themeDir = $this->config['themeDir'] . DIRECTORY_SEPARATOR . $theme[0]['value'] . DIRECTORY_SEPARATOR . "theme.html";
			$this->templateName = $theme[0]['value'];
		}
		else
		{
			$themeDir = $this->config['themeDir'] . DIRECTORY_SEPARATOR . $this->config['defaultTheme'] . DIRECTORY_SEPARATOR . "theme.html";
			$this->templateName = $this->config['defaultTheme'];
		}
		$handle = fopen ($themeDir, "r");
		$this->template = fread ($handle, filesize ($themeDir));
		fclose ($handle);
	}

	/**
	 * @internal
	 */
	public function onDestruct()
	{
		$this->flushContent();
		ob_end_clean();
		ob_start();
		echo $this->template;
		ob_flush();
		ob_flush();
	}

	/**
	 * Creates init
	 */
	private function createInit()
	{
		if($this->initRun)
		{
			$mainMod = explode("/", OpenVCP::calledMod(true));
			$dir = dirname($mainMod[1] ? str_replace("/".$mainMod[1], "", OpenVCP::calledMod()) : OpenVCP::calledMod());
			$dp  = opendir($dir);
			if(!preg_match('/.*\\\mods\\\error/', $dir)){
				$auth = & OpenVCP::call('Auth', 'getReference');
				if(file_exists($dir . DIRECTORY_SEPARATOR . 'locale'))
				{
					if(!is_array(($info = @parse_ini_file($dir . DIRECTORY_SEPARATOR . 'locale', true))))
					$info = array();
				}
				else
				$info = array();
				$homeDir = OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . (OPENVCP_MODS_DIR == dirname(dirname(OpenVCP::calledMod())) ? "" : DIRECTORY_SEPARATOR), '', dirname(dirname(OpenVCP::calledMod())))));
				preg_match("/openvcp\/(.*)/i", $homeDir, $subpattern);
				if($subpattern[1] == "")
				{
					if($auth->user_information['is_admin'])
					{
						$homeDir .= "super";
					}
					else
					{
						$homeDir .= "user";
					}
				}
				$this->headerMenu  = "<li><a href=\"" . $homeDir . "\"> " . _("home") . "</a></li>";

				while($f = readdir($dp)){
					$folders[] = $f;
				}

				sort($folders);
				foreach($folders as $key => $f)
				{
					$fullPath = $dir . DIRECTORY_SEPARATOR . $f;
					if($f !== '.' && $f !== '..' && !preg_match('/^\..*$/', $f)){
						if(is_dir($fullPath)){
							$this->headerMenu  .= "<li".(($mainMod[1] && $mainMod[1] == $f) ? " id=active" : "")."><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $fullPath))) . "\"> " . _($f) . "</a></li>";
						}
						else if(preg_match('/(.*)\.php/', $f, $sub) && $f !== 'main.php' && $f != 'functions.php'){
							$f = $sub[1];
							if(key_exists(OpenVCP::currentLocale(), (is_array($info["file:$f"]) ? $info["file:$f"] : array()))){
								$this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', str_replace('.php', '', $fullPath)))) . "\"> " . $info["file:$f"][OpenVCP::currentLocale()] . "</a></li>";
							}
							else $this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', str_replace('.php', '', $fullPath)))) . "\"> " . $f . "</a></li>";
						}
					}
				}
				closedir($dp);
				$this->info = $info;
				$this->createContentMenu($this->info, dirname(OpenVCP::calledMod()));

				//get css file
				$db =& OpenVCP::call('DB', 'getReference');
				if($auth->user_information['is_admin'])
				{
					$theme = $db->select($db->table_admin_settings, NULL, array('value'), array('adminid' => $auth->user_information['id'], 'AND', 'key' => 'theme'));
				}
				else
				{
					$theme = $db->select($db->table_user_settings, NULL, array('value'), array('userid' => $auth->user_information['id'], 'AND', 'key' => 'theme'));
				}
				if($theme != null)
				{
					$cssFile 	= OpenVCP::getWebPath(). DIRECTORY_SEPARATOR .$this->config['relPath'] . DIRECTORY_SEPARATOR . $theme[0]['value'] . '/theme.css';
				}
				else
				{
					$cssFile 	= OpenVCP::getWebPath(). DIRECTORY_SEPARATOR .$this->config['relPath'] . DIRECTORY_SEPARATOR . $this->config['defaultTheme'] . '/theme.css';
				}

				if($this->config['debugMode'] != '0')
				{
					$debugContent = ob_get_clean();
					if($this->config['debugMode'] == '2' && is_string($debugContent)){
						$this->template = str_replace('{DEBUG}'	, "<textarea id=\"debugScreen\">" . strip_tags($debugContent) . "</textarea><br/><br/>", $this->template);
					}
					else if($this->config['debugMode'] == '1' && in_array('debug', OpenVCP::getParams()) && is_string($debugContent)){
						$this->template = str_replace('{DEBUG}'	, "<textarea id=\"debugScreen\">" . strip_tags($debugContent) . "</textarea><br/><br/>", $this->template);
					}
					else
					{
						//unset($debugContent);
						$this->template = str_replace('{DEBUG}'	, "<!-- DEBUG -->", $this->template);
					}
				}
				else
				{
					ob_end_clean();
					$this->template = str_replace('{DEBUG}'	, "<!-- DEBUG -->", $this->template);
				}

				$this->template = str_replace('{BANNER}'		, OpenVCP::getWebPath().DIRECTORY_SEPARATOR.$this->config['relPath'].DIRECTORY_SEPARATOR.$this->config['defaultTheme'].DIRECTORY_SEPARATOR."banner.png", $this->template);
				$this->template = str_replace('{CONTENTMENU}', "<ul>" . $this->contentMenu . "</ul>", $this->template);
				$this->template = str_replace('{HEADERMENU}' , "<ul>" . $this->headerMenu  . "</ul>", $this->template);
				$this->template = str_replace('{TITLE}'  	, $this->pageTitle, $this->template);

				if(file_exists(str_replace("main.php", "main.js", OpenVCP::calledMod(false))))
				$this->template = str_replace('</title>', '</title><script type="text/javascript" src="' . OpenVCP::getWebPath(). DIRECTORY_SEPARATOR . "mods" . DIRECTORY_SEPARATOR . OpenVCP::calledMod(true) . DIRECTORY_SEPARATOR . "main.js\"></script>", $this->template);

				$this->template = str_replace('href="theme.css"', 'href="' . $cssFile . '"', $this->template);
				if($this->config['debugMode'] == 2 || $this->config['debugMode'] == 1 && in_array('debug', OpenVCP::getParams()))
				preg_match("/(.*)\<\/textarea\>/is", $this->template, $result);
				else
				preg_match("/(.*)<!-- DEBUG -->/is", $this->template, $result);
				ob_start();
				echo $result[0];
				unset($result);
				ob_end_flush();
				ob_start();
				if($this->config['debugMode'] == 2 || $this->config['debugMode'] == 1 && in_array('debug', OpenVCP::getParams()))
				$result = preg_split('/(.*)\<\/textarea\>/is', $this->template, -1, PREG_SPLIT_NO_EMPTY);
				else
				$result = preg_split("/(.*)<!-- DEBUG -->/is", $this->template, -1, PREG_SPLIT_NO_EMPTY);
				$this->template = $result[0];
				$this->template = preg_replace("/\{[CONTENT]+\}/s", "", $this->template);
				unset($result);
			}
			$this->initRun = false;
		}
	}

	/**
	 * flush's the current content
	 */

	public function flushContent()
	{
		if($this->initRun)
		{
			$this->createInit();
		}

		if($this->config['debugMode'] != '0')
		{
			$debugContent = ob_get_clean();
			if($debugContent != "")
			{
				if($this->config['debugMode'] == '2' && is_string($debugContent)){
					echo "<textarea id=\"debugScreen\">" . strip_tags($debugContent) . "</textarea><br/><br/>";
				}
				else if($this->config['debugMode'] == '1' && in_array('debug', OpenVCP::getParams()) && is_string($debugContent)){
					echo "<textarea id=\"debugScreen\">" . strip_tags($debugContent) . "</textarea><br/><br/>";
				}
				else unset($debugContent);
			}
		}
		else ob_end_clean();
		$this->contentData = preg_replace("/\{[eId]+[0-9]+\}/s", "", $this->contentData);
		echo $this->contentData;
		flush();
		ob_end_flush();
		ob_start();
		ob_start();
		$this->resetContent();
	}

	/**
	 * Creates the contentmenu
	 * @param array $info
	 * @param string $dir
	 * @internal
	 */
	private function createContentMenu($info = array(), $dir)
	{
		foreach($info as $key => $var)
		{
			$n = explode(':', $key);
			if($n[0] === 'link')
			{
				if(key_exists(OpenVCP::currentLocale(), $var))
				{
					$this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $dir))) . '/' . $n[1] . "\"> " . $var[OpenVCP::currentLocale()] . "</a></li>\n";
				}
				else if(key_exists('en_EN', $var))
				{
					$this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $dir))) . '/' . $n[1] . "\"> " . $var['en_EN'] . "</a></li>\n";
				}
				else
				{
					$this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $dir))) . '/' . $n[1] . "\"> " . $n[1] . "</a></li>\n";
				}
			}
			else continue;
		}
		if(count($this->contentMenuAdd)>0)
		{
			foreach($this->contentMenuAdd as $key => $var)
			{
				if(key_exists(OpenVCP::currentLocale(), $var))
				{
					$this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $dir))) . '/' . $key . "\"> " . $var[OpenVCP::currentLocale()] . "</a></li>\n";
				}
				else if(key_exists('sub', $var))
				{
					$this->contentMenu  .= "<div ".$var['sub']."><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $dir))) . '/' . $key . "\"> " . $var['en_EN'] . "</a></div>\n";
				}
				else if(key_exists('en_EN', $var))
				{
					$this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $dir))) . '/' . $key . "\"> " . _($var['en_EN']) . "</a></li>\n";
				}
				else if(key_exists('hr', $var))
				{
					$this->contentMenu .= "<hr>\n";
				}
				else if(key_exists('br', $var))
				{
					$this->contentMenu .= "<br>\n";
				}
				else if(key_exists('head', $var))
				{
					$this->contentMenu .= $var['head']."\n";
				}
				else
				{
					$this->contentMenu  .= "<li><a href=\"" . OpenVCP::makeURL(str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', $dir))) . '/' . $key . "\"> " . $var . "</a></li>\n";
				}
			}
		}
		return true;
	}

	/**
	 * Adds extra content to the content menu
	 * @param string $url
	 * @param string $urltext
	 */
	public function addContentMenu($url,$urltext)
	{
		$this->contentMenuAdd[$url] = array("en_EN" => $urltext);
		return true;
	}
	 
	/**
	 * Adds a line into the content Menu
	 */
	public function addContentMenuLine()
	{
		$this->contentMenuAdd[] = array("hr" => "<hr>");
		return true;
	}

	/**
	 * Adds some free space into the content Menu
	 */
	public function addContentMenuSpace()
	{
		$this->contentMenuAdd[] = array("br" => "<br>");
		return true;
	}

	/**
	 * Adds a heading to the content Menu
	 *
	 * @param string $text
	 */
	public function addContentMenuHead($text, $size = 2, $margin = 0)
	{
		$this->contentMenuAdd[] = array('head' => "<h" . $size . " class=\"subMenu$margin\">$text</h" . $size . ">");
		return true;
	}

	/**
	 * Adds a indented into the content Menu
	 *
	 * @param string $url
	 * @param string $urltext
	 */
	public function addContentMenuSub($url, $urltext, $margin = 1)
	{
		$this->contentMenuAdd[$url] = array("en_EN" => "$urltext", "sub" => "class=\"subMenu$margin\"");
		return true;
	}

	/**
	 * Append html data to the
	 * content frame
	 *
	 * @param string $data
	 */
	public function content($data){
		$this->contentData .= $data;
	}

	/**
	 * Resets the content html
	 * data
	 *
	 */
	public function resetContent(){
		$this->contentData = "";
	}

	/**
	 * @return elementId
	 */
	private function createElementId()
	{
		$this->elementId = $this->elementId + 200;
		return $this->elementId;
	}

	/**
	 * Creates a h tag with the given
	 * text.
	 *
	 * @param  string $text
	 * @param  string $type 1 or 2
	 * @param  int	  $element
	 * @return string
	 */
	public function h($text, $type = '1', $element = 0){
		$type = $type === '1' ? '1' : '2';
		if($element == 0)
		$this->contentData .=  "<h$type>" . nl2br(htmlentities($text)) . "</h$type>\n";
		else
		$this->contentData = str_replace("{eId".$element."}", "<h".$type.">" . nl2br(htmlentities($text)) . "</h".$type.">\n{eId".$element."}", $this->contentData);
	}

	/**
	 * Creates a  p with the given
	 * text.
	 * @param string $text
	 * @param int $element
	 * @param bool   $htmlentities
	 * @param string $id
	 * @return string
	 */
	public function p($text, $element = 0, $htmlentities = true, $id = "")
	{
		if($element == 0)
		$this->contentData .= "<p" . (($id != "") ? " id=".$id : "") . ">" . nl2br(($htmlentities ? htmlentities($text) : $text)) . "</p>\n";
		else
		{
			if($text == "")
			{
				$id = $this->createElementID();
				$this->contentData .= "<p".($id != "" ? " id=".$id : "").">{eId" . $id . "}</p>\n";
				return $id;
			}
			else
			{
				$id = $this->createElementID();
				$this->contentData = str_replace("{eId" . $element . "}", "<p" . (($id != "") ? " id=".$id : "").">" . $text . "{eId" . $id . "}</p>\n{eId" . $element . "}", $this->contentData);
				return $id;
			}
		}
	}

	/**
	 * Creates a span tag with the given
	 * text.
	 * @param string $text
	 * @param bool   $htmlentities
	 * @return string
	 */
	public function span($text, $htmlentities = true)
	{
		if($text != "")
		$this->contentData .= "<span>" . nl2br(($htmlentities ? htmlentities($text) : $text)) . "</span>\n";
		else
		{
			$id = $this->createElementID();
			$this->contentData .= "<span>{eId" . $id . "}</span>\n";
			return $id;
		}
	}

	public function img($element, $name, $mod = true, $alt = "", $height = 0, $width = 0)
	{
		if($mod)
		$this->contentData = str_replace("{eId" . $element . "}", "<img src=" . OpenVCP::getWebPath() . DIRECTORY_SEPARATOR . $this->config['relPath'].DIRECTORY_SEPARATOR.$this->templateName.DIRECTORY_SEPARATOR.$name.($alt != "" ? " alt=\"" . $alt . "\" name=\"" . $alt . "\"" : "").($width > 0 ? " width=\"". $width . "\" height=\"" . $height . "\"" : "")." border=\"0\">{eId" . $element . "}", $this->contentData);
		else
		{
			preg_match("/[a-z]+.php/is", str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', OpenVCP::calledMod())), $subpattern);
			$this->contentData = str_replace("{eId" . $element . "}", "<img src=" . OpenVCP::getWebPath() . DIRECTORY_SEPARATOR . "openvcp" . DIRECTORY_SEPARATOR . str_replace("/".$subpattern[0], '', str_replace('\\', '/', str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', OpenVCP::calledMod())))."/graph/" . $name . ">{eId" . $element . "}", $this->contentData);
		}
	}

	/**
	 * Creates a div tag
	 *
	 * @param string $text
	 * @param string $id
	 */
	public function div($text, $id, $element = 0, $class = "", $htmlentities = true)
	{
		if($text != "")
		$this->contentData .= "<div id=" . $id . " class=\"" . $class . "\">" . nl2br(($htmlentities ? htmlentities($text) : $text)) . "</div>\n";
		else
		{
			$elementId = $this->createElementID();
			if($element == 0)
			{
				$this->contentData .= "<div id=" . $id . " class=\"" . $class . "\">{eId" . $elementId . "}</div>\n";

			}
			else
			{
				$this->contentData = str_replace("{eId" . $element . "}", "<div id=" . $id . " class=\"" . $class . "\">{eId" . $elementId . "}</div>\n{eId" . $element . "}", $this->contentData);
			}
			return $elementId;
		}
	}

	/**
	 * Creates a < br > tag
	 *
	 * @param int $element
	 */
	public function br($element = 0)
	{
		($element == 0) ? $this->contentData .= "\n<br>\n" : $this->contentData = str_replace("{eId" . $element . "}", "<br>{eId" . $element . "}", $this->contentData);
	}

	/**
	 * Adds content to an element
	 *
	 * @param int $element
	 * @param string $content
	 */
	public function addToElementID($element, $content)
	{
		$this->contentData = str_replace("{eId" . $element . "}", $content . "{eId" . $element . "}", $this->contentData);
	}

	/**
	 * Sets the page title
	 *
	 * @param string $text
	 */
	public function title($text){
		$this->pageTitle = htmlentities($text);
	}

	/**
	 * Creates a new form and returns
	 * the id to work with it.
	 *
	 * @param int $element
	 * @param string $name
	 * @param string $action
	 * @param string $method
	 * @return int
	 */
	public function form($element = 0, $name, $action, $method = "POST"){
		$id = $this->createElementId();
		if($element == 0)
			$this->contentData .= "<form name=\"" . $name . "\" action=\"" . $action . "\" method=\"" . $method . "\">{eId" . $id . "}</form>\n";
		else
			$this->contentData = str_replace("{eId" . $element . "}","<form name=\"" . $name . "\" action=\"" . $action . "\" method=\"" . $method . "\">{eId" . $id . "}</form>{eId" . $element . "}\n", $this->contentData);
		return $id;
	}

	/**
	 * Adds a new input element to the
	 * given form.
	 *
	 * @param int $element
	 * @param string $type
	 * @param string $name
	 * @param string $value
	 * @param array $params
	 * @param string $function
	 * @return bool
	 */
	public function formAddInputElement($element, $type, $name, $value = "", $params = array(), $id = "", $class = "", $function = "")
	{
		$inputbox = "<input type=\"" . $type . "\" name=\"" . $name . "\" ";
		if($params != null)
		{
			foreach($params as $key => $value)
			{
				$inputbox .= "$key=\"$value\" ";
			}
		}
		$inputbox .= "value=\"$value\"";
		if($id != "")
		$inputbox .= " id=\"" . $id . "\"";
		if($class != "")
		$inputbox .= " class=\"" . $class . "\"";
		if($function != "")
		$inputbox .= " " . $function;
		$inputbox .= ">{eId" . $element . "}\n";
		$this->contentData = str_replace("{eId" . $element . "}", $inputbox, $this->contentData);
		return true;
	}

	public function formAddButton($element, $name, $value, $onclickaction = "")
	{
		$button = "<input type=\"button\" name=\"" . $name . "\" value=\"" . $value . "\"" . ($onclickaction == "" ? "" : " onclick=\"" . $onclickaction . "\"").">{eId" . $element . "}\n";
		$this->contentData = str_replace("{eId" . $element . "}", $button, $this->contentData);
		return true;
	}

	/**
	 * Adds a textarea into a form
	 *
	 * @param int $element
	 * @param string $name
	 * @param string $value
	 * @param array $params
	 * @return bool
	 */
	public function formAddTextarea($element, $name, $value = "", $params = array())
	{
		$textarea = "<textarea ";
		foreach(@$params as $key => $value_param)
		{
			$textarea .= $key . "=\"" . $value_param . "\" ";
		}
		$textarea .= " name=\"" . $name . "\">" . $value . "</textarea>\n";
		$this->contentData = str_replace("{eId" . $element . "}", $textarea, $this->contentData);
		return true;
	}

	/**
	 * Adds a dropdownbox into a elementid
	 *
	 * @param int $element
	 * @param string $name
	 * @param array $contents
	 * @param string $selected that key of contents which should be selected
	 * @param int $size
	 * @param string $function
	 */
	public function formAddDropdown($element, $name, $contents = array(), $selected = "", $size = 5, $id = "", $class = "", $function = "")
	{
		$select  = "<select name=\"$name\"";
		if($id != "")
		$select .= " id=\"" . $id . "\"";
		if($class != "")
		$select .= " class=\"" . $class . "\"";
		if($function != "")
		$select .= " ".$function;
		$select .= ">";
		if(is_array($contents))
		{
			foreach($contents as $key => $value)
			{
				$select .= $value == $selected ? "<option value=\"" . $value . "\" selected>" . $key . "</option>\n" : "<option value=\"" . $value . "\">" . $key . "</option>\n";
			}
		}
		$select .= "</select>\n";
		$this->contentData = str_replace("{eId" . $element . "}",$select . "{eId" . $element . "}", $this->contentData);
		return true;
	}

	/**
	 * Adds a checkbox/radio into the elementid
	 *
	 * @param int $element
	 * @param string $type
	 * @param string $name
	 * @param string $value
	 * @param string $function
	 * @param bool $checked
	 */
	public function formAddCheckElement($element, $type, $name, $value, $checked = false, $function = "")
	{
		if($type == "checkbox" || $type == "radio")
		{
			$checkbox = "<input type=\"" . $type . "\" name=\"" . $name . "\" value=\"" . $value . "\"". ($checked ? " checked" : "") . ($function != "" ? " " . $function : "") . ">\n";
			$this->contentData = str_replace("{eId" . $element . "}",$checkbox."{eId" . $element . "}", $this->contentData);
			return true;
		}
		else return false;
	}

	/**
	 * Adds a fielset
	 *
	 * @param string $legend
	 *
	 * @return int $elementId
	 */
	public function fieldset($legend)
	{
		$id = $this->createElementId();
		$this->contentData .= "<fieldset><legend>" . $legend . "</legend>{eId" . $id . "}</fieldset>";
		return $id;
	}

	/**
	 * Adds a new table
	 *
	 * @param int $element
	 * @param string $valign
	 * @param int $cellspacing
	 * @param int $cellpadding
	 * @param string $width
	 * @param int $height
	 * @return id of element
	 */
	public function table($element = 0, $valign = "top", $cellspacing = 2, $cellpadding = 2, $width = "*", $height = 0, $did = "")
	{
		$id = $this->createElementId();
		if($element == 0)
		$this->contentData .= "<table class=\"normaltable\" valign=\"" . $valign . "\" cellpadding=\"" . $cellpadding . "\" cellspacing=\"" . $cellspacing . "\" width=\"" . $width . "\"" . ($height == 0 ? "" : " height=\"" . $height . "\"") . ($id != "" ? " id=\"" .OpenVCP::calledMod(true).$id . "\"" : "") . ">{eId" . $id . "}</table>\n";
		else
		$this->contentData = str_replace("{eId" . $element . "}", "<table class=\"normaltable\" valign=\"" . $valign . "\" cellpadding=\"" . $cellpadding . "\" cellspacing=\"" . $cellspacing . "\" width=\"" . $width . "\"". ($height == 0 ? "" : " height=\"" . $height . "\"") .($id != "" ? " id=\"" .OpenVCP::calledMod(true).$id . "\"" : "") . ">{eId" . $id . "}</table>{eId" . $element . "}\n", $this->contentData);
		$this->tableRowCounter[$id] = 0;
		return $id;
	}

	/**
	 * Adds a row into a table
	 *
	 * @param int $element
	 * @param string $valign
	 * @param string $width
	 * @param int $height
	 * @return $elementId of row
	 */
	public function tableAddRow($element, $valign="top", $width = "*", $height = 0)
	{
		$id = $this->createElementId();
		$this->contentData = str_replace("{eId" . $element . "}", "<tr valign=\"" . $valign . "\" width=\"" . $width . "\"". ($height == 0 ? "" : " height=\"" . $height . "\"") . (($this->tableRowCounter[$element] % 2 == 0 ? " class=\"tr1\"" : " class=\"tr2\"")).">{eId" . $id . "}</tr>\n{eId" . $element . "}", $this->contentData);
		$this->tableRowCounter[$element]++;
		return $id;
	}

	/**
	 * Adds a row into a table
	 *
	 * @param int $element
	 * @param string $valign
	 * @param string $width
	 * @param int $height
	 * @return $elementId of row
	 */
	public function tableAddHead($element, $valign="top", $width = "*", $height = 0)
	{
		$id = $this->createElementId();
		$this->contentData = str_replace("{eId" . $element . "}", "<thead valign=\"" . $valign . "\" width=\"" . $width . "\"". ($height == 0 ? "" : " height=\"" . $height . "\"") .">{eId" . $id . "}</thead>\n{eId" . $element . "}", $this->contentData);
		$this->tableRowCounter[$id] = 1;
		return $id;
	}
	 
	/**
	 * Adds a col into a row
	 * @param int $element
	 * @param string $content
	 * @param int $colspan
	 * @param string $width
	 * @param int $height
	 * @return $elementId of col
	 */
	public function tableAddCol($element, $content = "", $colspan = 1, $width = "*", $height = 0, $align="left")
	{
		$id = $this->createElementId();
		$this->contentData = str_replace("{eId" . $element . "}", "<td align=\"".$align."\" width=\"" . $width . "\"". ($height == 0 ? "" : " height=\"" . $height . "\"") ." colspan=\"" . $colspan . "\">" . $content . "{eId" . $id . "}</td>{eId" . $element . "}", $this->contentData);
		return $id;
	}

	/**
	 * Adds and creates a link into
	 * the given element
	 * @param int $element
	 * @param string $url
	 * @param string $urltext
	 */
	public function a($element, $url, $urltext, $target = "")
	{
		$eId = $this->createElementId();
		if($element == 0)
			$this->contentData .= "<a href=\"".OpenVCP::makeURL($url)."\"" . (($target != "") ? " target=\"" . $target . "\"" : "") . ">".$urltext."{eId" . $eId . "}</a>";
		else
			$this->contentData = str_replace("{eId" . $element . "}", "<a href=\"".OpenVCP::makeURL($url)."\"" . (($target != "") ? " target=\"" . $target . "\"" : "") . ">".$urltext."{eId" . $eId . "}</a>{eId" . $element . "}", $this->contentData);
		return $eId;
	}
	/**
	 * Marks a text in a different color
	 *
	 * @param string $text
	 * @param string $color
	 * @return string fonttag
	 */
	public function markUp($text, $color = "red")
	{
		return "<font color=\"" . $color . "\">" . $text . "</font>";
	}

	/**
	 * Returns the theme dir
	 */
	public function getThemeDir()
	{
		return $this->config['themeDir'];
	}
}
?>
