<?php
/**
 * OpenVCP GUI Core Extension
 * This extension is a built in one,
 * so there no need to load it anywhere.
 * It provides methods to create
 * the output of the interface.
 *
 * @author  Oliver Werner
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

final class OpenVCP_GRAPH extends OpenVCP_Core {
	
	/**
	 * Core module version
	 *
	 * @var string
	 */
	protected $version = '0.3.0.1';
	
	/**
	 * Allow references
	 * 
	 * @var bool
	 */
	protected $getRef = true;

	/**
	 * Holds the image
	 */
	private $image;
	
	/**
	 * Holds the colors
	 */
	private $color = array();

	public function onConstruct()
	{
		ob_start();
		header("Content-type: image/png");
		if(!function_exists(ImageCreate))
			return;
		$this->image = ImageCreate($this->config['width'], $this->config['height']) or die ("Couldn't create GD-Stream");
		$background_color = ImageColorAllocate ($this->image, 255, 255, 255);
	}
	
	public function onDestruct()
	{
		ob_end_clean();
		ImagePNG($this->image);
	}
	
	/**
	 * set's the color to the image format
	 * @param int red
	 * @param int green
	 * @param int blue
	 * @return int the class intern number of the color
	 */
	public function setColor($red, $green, $blue)
	{
//		if($red >= 0 && $red >= 255 && $green >= 0 && $green >= 255 && $blue >= 0 && $blue >= 255)
//		{
			$colorIndex = count($this->color);
			$this->color[$colorIndex] = ImageColorAllocate ($this->image, $red, $green, $blue);
			return $colorIndex;
//		}
//		else
	//		return 0;
	}
	
	/**
	 * creates a new line into the graph
	 * @param int $x1 left upper corner of the line
	 * @param int $y1 left upper corner of the line
	 * @param int $x2 right under corner of the line
	 * @param int $y2 right under corner of the line
	 * @param int $color the intern color
	 */
	public function newLine($x1, $y1, $x2, $y2, $lineColor)
	{
		ImageLine($this->image, $x1, $y1, $x2, $y2, $this->color[$lineColor]);
	}
	
	/**
	 * creates a new string into the graph
	 * @param int $font set's the intern font size
	 * @param int $x set's the x coordinate
	 * @param int $y set's the y coordinate
	 * @param string $string the string to add
	 * @param int $color the intern color
	 */
	public function newString($font, $x, $y, $string, $color)
	{
		imagestring($this->image, $font, $x, $y, $string, $this->color[$color]);
	}
	
	/**
	 * creates a new filled ellipse into the graph
	 * bool imagefilledellipse ( resource image, int cx, int cy, int w, int h, int color )
	 * @param int $centerX
	 * @param int $centerY
	 * @param int $width
	 * @param int $height
	 * @param int $color
	 */
	public function newFilledEllipse($centerX, $centerY, $width, $height, $color)
	{
		imagefilledellipse($this->image, $centerX, $centerY, $width, $height, $this->color[$color]);
	}

	/**
	 * creates a new ellipse into the graph
	 * bool imagefilledellipse ( resource image, int cx, int cy, int w, int h, int color )
	 * @param int $centerX
	 * @param int $centerY
	 * @param int $width
	 * @param int $height
	 * @param int $color
	 */
	public function newEllipse($centerX, $centerY, $width, $height, $color)
	{
		imageellipse($this->image, $centerX, $centerY, $width, $height, $this->color[$color]);
	}
		
	/**
	 * creates a new filles rectangle into the graph
	 * @param int $x1 left upper corner of the rectangle
	 * @param int $y1 left upper corner of the rectangle
	 * @param int $x2 right under corner of the rectangle
	 * @param int $y2 right under corner of the rectangle
	 * @param int $color
	 */
	public function newFilledRectangle($x1, $y1, $x2, $y2, $color)
	{
		imagefilledrectangle($this->image, $x1, $y1, $x2, $y2, $this->color[$color]);
	}
	
	/**
	 * creates a new rectangle into the graph
	 * @param int $x1 left upper corner of the rectangle
	 * @param int $y1 left upper corner of the rectangle
	 * @param int $x2 right under corner of the rectangle
	 * @param int $y2 right under corner of the rectangle
	 * @param int $color
	 */
	public function newRectangle($x1, $y1, $x2, $y2, $color)
	{
		imagerectangle($this->image, $x1, $y1, $x2, $y2, $this->color[$color]);
	}
	
	public function getWidth()
	{
		return($this->config['width']);
	}

	public function getHeight()
	{
		return($this->config['height']);
	}
}
?>
