<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  OpenVCP Team
 * @version 0.3.0.5
 * @package OpenVCP-v0.3
 */

### DEFINES (Filesystem)
define('OPENVCP_ROOT_DIR'	, realpath(dirname(__FILE__) . DIRECTORY_SEPARATOR . '..'));
define('OPENVCP_CORE_DIR'	, OPENVCP_ROOT_DIR . DIRECTORY_SEPARATOR . 'core');
define('OPENVCP_MODS_DIR'	, OPENVCP_ROOT_DIR . DIRECTORY_SEPARATOR . 'mods');
define('OPENVCP_CACHE_DIR'	, '/var/cache/openvcp/');

### DEFINES (Core)
define('OPENVCP_CORE_PREFIX', 'OpenVCP_');

### DEFINES (Config)
define('OPENVCP_CORE_CONFIG', '/etc/openvcp/openvcp.conf');

### DEFINES (Debug)
define('OPENVCP_E_QUIET'  , 0);
define('OPENVCP_E_WARNING', 1);
define('OPENVCP_E_ERROR'  , 2);

### SET INCLUDE DIRS
ini_set('include_path', ini_get('include_path') . PATH_SEPARATOR . OPENVCP_CORE_DIR);

### IMPORTANT INCLUDES
require_once('Config.php');
require_once('Cache/Lite.php');

/**
 * OpenVCP Core Class
 * This static class representates the Core
 * of the OpenVCP Interface SE. Every Action
 * inside OpenVCP and his Mods must be done
 * over this class.
 *
 * @author  Jean-Pascal Hoevel
 * @version 0.3.0.4
 * @package OpenVCP-Interface-0.3-SE 
 */
final class OpenVCP {
	
	/**
	 * Contains the loaded core
	 * objects.
	 *
	 * @var array
	 */
	protected static $loadedCore = array();
		
	/**
	 * Contains the config object
	 * for the class self
	 *
	 * @var Config_Container
	 */
	protected static $coreConfig = null;
		
	/**
	 * This var will make sure,
	 * that the main function are
	 * only called once.
	 *
	 * @var bool
	 */
	protected static $mainCalled = false;
	
	/**
	 * Contains the params who passed
	 * the main call
	 *
	 * @var array
	 */
	protected static $passedParams = array();
	
	/**
	 * Saves the path to the 
	 * gettext locale dir
	 *
	 * @var string
	 */
	public static $localeDir = "";
	
	/**
	 * Contains the id of the
	 * current locale (e.g. en_GB)
	 *
	 * @var string
	 */
	protected static $currentLocale = "";
	
	/**
	 * Contains the webpath
	 * to OpenVCP relative from
	 * document root.
	 *
	 * @var string
	 */
	protected static $webPath = "";
	
	/**
	 * Contains the last error
	 * message
	 *
	 * @var string
	 */
	protected static $lastError = null;
	
	/**
	 * Contains the path to the
	 * called mod page
	 *
	 * @var string
	 */
	protected static $calledMod = "";
	
	/**
	 * contains the config
	 */
	protected static $mainConfig = null;
	/**
	 * The main function called in OpenVCP.php
	 * index file. 
	 * 
	 * This function shall do all
	 * needed things depend on the request.
	 *
	 * @return void
	 */
	public static function main(){
		if(!self::$mainCalled)
		{
			//TODO: Avoid XSS better
			foreach($_POST as $key => $value) {
				if(is_array($value))
					foreach($value as $subkey => $subvalue) {
						if(is_array($subvalue))
							foreach($subvalue as $subsubkey => $subsubvalue)
								$_POST[$key][$subkey][$subsubkey] = htmlspecialchars(strip_tags($subsubvalue));
						else
							$_POST[$key][$subkey] = htmlspecialchars(strip_tags($subvalue));
					}
				else
					$_POST[$key] = htmlspecialchars(strip_tags($value));
			}

			self::$mainCalled = true;
			$mainConfig = self::_getCoreConfig('OpenVCP');
			self::$mainConfig = $mainConfig;
			self::$localeDir = $mainConfig['localeDir'];
			self::$webPath   = $mainConfig['webPath'];
			
			self::$currentLocale = $mainConfig['defaultLocale'];
			@setlocale(LC_MESSAGES, $mainConfig['defaultLocale']);
			putenv("LANG=" . $mainConfig['defaultLocale']);
			bindtextdomain('core', $mainConfig['localeDir']);
			bind_textdomain_codeset('core', 'iso-8859-15');
			register_shutdown_function(array(OpenVCP, 'shutdown'));
			$call    = preg_replace('/\//', DIRECTORY_SEPARATOR, preg_replace('/^.*?(openvcp\/|index\.php\/)\/?(.*)$/', '\\2', $_SERVER['REQUEST_URI']));
			$call 	 = str_replace('index.php', '', $call);
			$callTmp = $call;
			if($call !== "")
			{
				if(strstr($call, "ajax"))
				{
					$call = trim($call, "ajax/");
				}
				else if(strstr($call, "graph"))
				{
					$call = trim($call, "graph/");
				}
				while(!file_exists(($file = OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR . $call . '.php')) && !file_exists(($file = OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR . $call . DIRECTORY_SEPARATOR . 'main.php')))
				{
					preg_match('/^(.*)' . (DIRECTORY_SEPARATOR === '/' ? '\/' : '\\\\') . '(.*)$/', $call, $sub);
					$call 	   = $sub[1];
					$params[]  = $sub[2];
					if($call == "") self::gotoURL('auth');
				}
				self::$passedParams = is_array($params) ? array_reverse($params) : array();
				
				self::$calledMod = $file;

				if(is_array($mainConfig['load']))
				{
					foreach($mainConfig['load'] as $load)
					{
						if(strstr($callTmp, "ajax") && $load == "GUI")
							self::_loadCoreClass("AJAX");
						elseif(strstr($callTmp, "graph") && $load == "GUI")
							self::_loadCoreClass("Graph");
						else
							self::_loadCoreClass($load);
					}
				}
				else if(is_string($mainConfig['load']))
				{
					self::_loadCoreClass($mainConfig['load']);
				}
				$auth = & self::call('Auth', 'getReference');
				$db =& self::call('DB', 'getReference');
				$settings = $db->select($db->table_user_settings, NULL, array('value'), array('userid' => $auth->user_information['id'], 'AND', 'key' => 'language'));
				$mod = explode(DIRECTORY_SEPARATOR, (str_replace(OPENVCP_MODS_DIR.DIRECTORY_SEPARATOR, '', $file)));
				$domain = "";
				foreach($mod as $sub)
				{
					$domain .= $sub ."_";
				}
				$domain = trim($domain, ".php_");
				if($settings != NULL)
				{
					setlocale(LC_ALL, $settings[0]['value']);
					putenv("LANG=".$settings[0]['value']);
				}
				else
				{
					setlocale(LC_ALL, $mainConfig['defaultLocale']);
					putenv("LANG=".$mainConfig['defaultLocale']);
				}
				bindtextdomain($domain, $mainConfig['localeDir']);
				#bind_textdomain_codeset($domain, 'iso-8859-1');
				textdomain($domain);
				
				$mainMod = explode("/", self::calledMod(true));
				$dir = dirname($mainMod[1] ? str_replace("/".$mainMod[1], "", self::calledMod()) : self::calledMod());
				if(file_exists($dir."/functions.php"))
					require_once($dir."/functions.php");
				if(strstr($callTmp, "ajax"))
					$file = str_replace("main.php", "ajax.php", $file);
				elseif(strstr($callTmp, "gragh"))
					$file = str_replace("main.php", "graph.php", $file);
				throw new Exception($file);
			}
			else self::gotoURL($mainConfig['defaultRedirect']);
		}
	}

	/**
	 * Calls a core method with the
	 * given parameter.
	 *
	 * @param string $module Name of the core module
	 * @param string $method Name of method to call
	 * @param array	 $params Array of params
	 * @return mixed
	 */
	public static function &call($module, $method, array $params = null){
		self::_loadCoreClass($module);
		if(method_exists(self::$loadedCore[$module], $method)){
			return self::$loadedCore[$module]->$method($params);
		}
		else self::coreError(_("The core class %1% does not provide a method named %2%."), 3, $module, $method);
	}
		
	/**
	 * Returns the params who passed
	 * the main call.
	 *
	 * @return array
	 */
	public static function getParams(){
		return self::$passedParams;
	}
	
	/**
	 * Redirects the client to another
	 * Modpage.
	 *
	 * @param string $url
	 */
	public static function gotoURL($url){
		ob_end_clean();
		header('Location: ' . self::makeURL($url));
		die();
	}
	
	/**
	 * Generates a valid URL for
	 * OpenVCP pages by the given
	 * relativ url.
	 *
	 * @param string $relURL
	 * @return string
	 */
	public static function makeURL($relURL){
		return self::$webPath . "/openvcp/$relURL";
	}
	
	/**
	 * Returns the webPath
	 */
	 public static function getWebPath()
	 {
	 	return self::$webPath;
	 }
	
	/**
	 * Returns the current locale
	 *
	 * @return string
	 */
	public static function currentLocale(){
		return self::$currentLocale;
	}
	
	/**
	 * Returns the path to the 
	 * called mod page.
	 *
	 * @return string
	 */
	public static function calledMod($relative = false){
		return ($relative === true ? dirname(str_replace(OPENVCP_MODS_DIR . DIRECTORY_SEPARATOR, '', self::$calledMod)) : self::$calledMod);
	}
		
	/**
	 * This function will be used by core
	 * modules.
	 *
	 * @param string $debugStr ID of the debug string in debug.conf
	 */
	public static function coreError($debugStr, $debugLevel = 2){
		try {
			throw new Exception();
		}
		catch(Exception $e){
			$e = $e->getTrace();
			$module = str_replace(OPENVCP_CORE_PREFIX, '', $e[1]['class']);
			$line   = $e[count($e)-2]['line'];
			$file	= str_replace(OPENVCP_ROOT_DIR . DIRECTORY_SEPARATOR, '', $e[count($e)-2]['file']);
			$method = $e[count($e)-2]['function'] . "()";
		}
		for($i = 2; $i < func_num_args(); $i++){
			$replace  = func_get_arg($i);
			$debugStr = str_replace('%' . ($i-1) . '%', $replace, $debugStr);
		}
		$debugStr .= " ($method, $file, #$line)";
		switch($debugLevel){
			case 0:
				break;
			case 1:
				echo "\nOpenVCP Warning: " . $debugStr . "\n\n";
				break;
			case 2:
				echo "\nOpenVCP Error: "   . $debugStr . "\n\n";
				die();
				break;
			default:
				echo "\nOpenVCP Error: "   . $debugStr . "\n\n";
				die();
				break;
		}
		self::$lastError = $debugStr;
	}
	
	/**
	 * Returns the last error message.
	 * If no error message set, the return
	 * value will be false.
	 *
	 * @return mixed
	 */
	public static function lastError(){
		return self::$lastError !== null ? self::$lastError : false;
	}
	
	/**
	 * returns the support mail address
	 */
	public static function supportMail()
	{
		return self::$mainConfig['supportMail'];
	}
	
	/**
	 * This function is used as getter
	 * for core classes. 
	 * 
	 * @param  string $module 
	 * @return void
	 */
	private static function _loadCoreClass($module){
		if(!key_exists($module, self::$loadedCore)){
			$modfile = OPENVCP_CORE_DIR . DIRECTORY_SEPARATOR . "$module.php";
			$modname = OPENVCP_CORE_PREFIX . $module;
			if(file_exists($modfile) && is_readable($modfile)){
				require_once($modfile);
				if(class_exists($modname)){
					self::_validateCoreClass($modname, $modfile);
					self::$loadedCore[$module] = new $modname(self::_getCoreConfig($module));
				}
				else self::coreError(_("The core class %1% could not be found in '%2%'."), 3, $modname, $modfile);
			}
			else self::coreError(_("The core class file %1% could not be found."), 3, $modfile);
		}
	}
	
	/**
	 * Gets the config of a core module.
	 *
	 * @param  string $module Name of the core module
	 * @return array
	 */
	private static function _getCoreConfig($module){
		if(is_null(self::$coreConfig)){
			$c = new Config;
			self::$coreConfig = $c->parseConfig(OPENVCP_CORE_CONFIG, 'apache');
			if(PEAR::isError(self::$coreConfig)){
				self::coreError(_("Failed to load the openvcp.conf core config file."));
			}
			else {
				$c = self::$coreConfig->getItem(null, $module);
				if(is_object($c)){
					$c = $c->toArray();
					$c = $c[$module];
					foreach($c as $var => $value){
						if($var !== '@'){
							$return[$var] = str_replace('%OPENVCP_ROOT%', OPENVCP_ROOT_DIR, $value);
						}
					}
					return $return;
				}
			}
		}
		else {
			$c = self::$coreConfig->getItem(null, $module);
			if(is_object($c)){
				$c = $c->toArray();
				$c = $c[$module];
				foreach($c as $var => $value){
					if($var !== '@'){
						$return[$var] = str_replace('%OPENVCP_ROOT%', OPENVCP_ROOT_DIR, $value);
					}
				}
				return $return;
			}
		}
	}
	
	/**
	 * Validates a core class.
	 * 
	 * On fail a exception will be thrown.
	 * Notice:
	 * The core class will only re-validated
	 * if the class file changed.
	 *
	 * @todo Add version\getRef vars etc check
	 * @param  string $class Name of the core class
	 * @return void
	 */
	private static function _validateCoreClass($class, $file){
		$c = new Cache_Lite(array('cacheDir' => OPENVCP_CACHE_DIR));
		if($c->get($class, 'validated_classes') !== sha1_file($file)){
			$r = new ReflectionClass($class);
			if(!$r->isSubclassOf('OpenVCP_Core')){
				self::coreError(_("The core class %1% must extend the OpenVCP base class (OpenVCP_Core)."), 3, $class);
			}
			else {
				if(!$r->hasProperty('version')){
					self::coreError(_("The core class %1% must provide the \$version."), 3, $class);
				}
				else {
					$prop = $r->getProperty('version');
					if(!$prop->isProtected()){
						self::coreError(_("The %1%'s version property must be protected."), 3, $class);
					}
				}
			}
			$c->save(sha1_file($file), $class, 'validated_classes');
		}
	}
	
	/**
	 * This function gonna be called when
	 * the user aborts the script on runtime.
	 * So it will be unset all loaded cores
	 * to call their onDestruct functions for
	 * a clean shutdown
	 */
	public static function shutdown(){
		foreach(self::$loadedCore as $core){
			unset($core);
		}
		return;
	}
}

/**
 * OpenVCP Core Core Class
 * This class will be extended by
 * the real core classes to make
 * use of its methods. The other
 * thing is, that the construct
 * and destruct of this class have
 * locked tasks, to keep the OpenVCP
 * process clean.
 * 
 * @author  Jean-Pascal Hoevel
 * @version 0.3.0.1
 */
class OpenVCP_Core {
	
	/**
	 * Contains the config array
	 * for the mod.
	 *
	 * @var array
	 */
	protected $config = array();
	
	/**
	 * This function will be called
	 * when OpenVCP creates a new core
	 * object. It can reveice a config
	 * when there's a config defined.
	 * 
	 * @param array $config
	 */
	final public function __construct(array $config = null){
		$this->config =& $config;
		if(method_exists($this, 'onConstruct')) $this->onConstruct();
	}
		
	/**
	 * This function will be called
	 * when the core object gonna destroyed.
	 */
	final public function __destruct(){
		if(method_exists($this, 'onDestruct')) $this->onDestruct();
	}
	
	/**
	 * Returns a reference
	 * when the core module set $getRef
	 * to true
	 *
	 * @return object
	 */
	final public function &getReference(){
		if(is_bool($this->getRef) && $this->getRef === true){
			return $this;
		}
		else OpenVCP::coreError(_("This core extension does not allow direct references."), 2);
	}
	
	/**
	 * Returns the version of the 
	 * core module.
	 *
	 * @return string
	 */
	final public function getVersion(){
		return $this->version;
	}
}
?>
