<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

require_once "Admin.class.php";
require_once "Net/SMTP.php";

class Sendmail {
	private $admin = null;
	private $mailReceiver = "";
	private $DB = null;
	private $mailSubject = "";
	private $mailContent = "";
	
	/**
	 * Creates a new Mail
	 *
	 * @param string $templateName
	 * @param int $adminid
	 */
	public function __construct($templateName, $adminid)
	{
		$this->DB =& OpenVCP::call('DB', 'getReference');
		$this->AUTH =& OpenVCP::call('Auth', 'getReference');
		$this->admin = new Admin($adminid);
		$content = $this->DB->select($this->DB->table_mail_templates, NULL, NULL, array('name' => $templateName, 'AND', 'adminid' => $this->admin->getId()));
		if(!is_array($content))
		{
			throw new exception (_("mail template does not exist")." \"".$templateName."\"");
			return;
		}
		$this->mailContent = $content[0]['content'];
		$this->mailSubject = $content[0]['subject'];
	}
	
	/**
	 * Replaces tags in the mail content
	 *
	 * @param array $content
	 */
	public function replaceContent(array $content)
	{
		preg_match_all('/%[a-zA-Z]+/', $this->mailContent, $returnPatterns);
		$thingsToReplace = explode(" ", str_replace("%", "", implode(" ", $returnPatterns[0])));
		foreach($thingsToReplace as $key => $value)
		{
			$this->mailContent = str_replace("%".$value, $content[$value], $this->mailContent);
		}
		return;
	}
	
	/**
	 * set's the mail receiver
	 */
	public function setTo($to)
	{
		$this->mailReceiver = $to;
	}
	
	/**
	 * adds the admins signature to the mail
	 * 
	 * @return boolean success
	 */
	private function _addSignatur()
	{
		$signature = $this->admin->getSetting("signature");
		if(is_string($signature))
		{
			$admin = $this->DB->select($this->DB->table_admin, NULL, NULL, array('id' => $this->admin->getId()));
			$this->mailContent .= "\n".$signature;
			$this->replaceContent($admin[0]);
			return true;
		}
		else
			return false;
	}
	
	public function sendMail()
	{
		$this->_addSignatur();
		if($this->admin->getSetting('mail') == "smtp")
		{
			$from = $this->admin->getEmailaddress();
			if (! ($smtp = new Net_SMTP($this->admin->getSetting("smtp_host")))) {
	    		die("Unable to instantiate Net_SMTP object\n");
			}
			
			$smtp->setDebug(true);
			
			if (PEAR::isError($e = $smtp->connect())) {
	    		die($e->getMessage() . "\n");
			}
	
			
			$ret = $smtp->auth($this->admin->getSetting("smtp_user"), $this->admin->getSetting("smtp_password"), $this->admin->getSetting("smtp_authtype"));
			
	
			if (PEAR::isError($ret))
			{
				throw new exception (_("Authentication with mailserver failed"));
			} 
			
			if (PEAR::isError($smtp->mailFrom($from))) {
				throw new exception (_("Unable to set sender to")."<$from>");
			}
	
			if (PEAR::isError($res = $smtp->rcptTo($this->mailReceiver)))
			{
				throw new exception (_("Unable to add recipient")."<".$this->receiver.">: " . $res->getMessage() . "\n");
			}
			echo "FROM: $from\n TO: ".$this->mailReceiver."\n";
		        if (PEAR::isError($smtp->data("From: <".$from.">\r\n".
					"To: <".$this->mailReceiver.">\r\n".
					"Subject: ".$this->mailSubject."\r\n".
					"Date: ".date("r")."\r\n\r\n".
					$this->mailContent))) {
		            die("Unable to send data\n");
		        }
	
			$smtp->disconnect();
		}
		else
		{
			mail($this->mailReceiver, $this->mailSubject, $this->mailContent, "From: ".$this->admin->getEmailaddress());
		}
	}
}
?>
