<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

/** 
 * OpenVCP Core - Socket Class 
 * Handels the connection to the nodes.
 * 
 * @author Oliver Werner / OpenVCP Team 
 * @version 0.3.0.1 
 * @package OpenVCP-v0.3 
 */ 
 
 
final class OpenVCP_Socket extends OpenVCP_Core {

	/**
	 * Core Module Version
	 *
	 * @var string
	 */
	protected $version = '0.3.0.1-alpha';

	/**
	 * This will allow
	 * mods to get a reference
	 * of this object
	 *
	 * @var bool
	 */
	protected $getRef = true;
	
	/**
	 * This will contain all existing sockets
	 * 
	 * @var array
	 */

	protected $sockets = array();

	/**
	 * not needed at the moment
	 */
	public function __onConstruct()
	{
		
	}
	
	/**
	 * closes all existing sockets
	 */
	 public function __onDestruct()
	 {
	 	isset($sockets);
	 	foreach(@$sockets as $key => $val)
	 	{
			if(isset($val['session']))
			 	gnutls_bye($val['session'], GNUTLS_SHUT_RDWR);

        	socket_close($val['socket']);
	 	}
	 }
	 

 	 /**
	  * establish a test connection to the given node
	  * 
	  * @param String $ip
	  * @param int $port
	  * @param String $user
	  * @param String $pw
	  * @param String $tls
	  */
	  public function testconnection($ip, $port, $user, $pw, $tls)
	  {

  		$errmsg =  "ok";
		$testsock = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);

		if($ip != "")
		{


	  		$time = time();
			socket_set_nonblock($testsock);
	
			while (!@socket_connect($testsock, $ip, $port))
	  		{
	     			$err = socket_last_error($testsock);

	     			if ($err == 115 || $err == 114)
	     			{
	       				if ((time() - $time) >= 10)
	       				{
	        				socket_close($testsock);
	        				return _('Couln\'t connect to node');
	       				}
		       		
					usleep(100);
	                    		continue;
     				}

				return _('Couln\'t connect to node');
	  		}
			socket_set_block($testsock);


			if($tls == 'y')
               		{
				if(function_exists('gnutls_session_new'))
				{
					socket_write($testsock, "TLS\n");
					$output = socket_read ($testsock, 10);
					$output = trim($output);

					if($output == "OK")
					{
						$session = gnutls_session_new(GNUTLS_CLIENT);
						$cred = gnutls_credentials_new(CRED_ANON_CLIENT);
						gnutls_set_default_priority($session);
						gnutls_kx_set_priority($session, GNUTLS_KX_ANON_DH);
						gnutls_session_set_credentials($session, $cred);
						gnutls_session_set_socket($session, $testsock);
						gnutls_handshake($session);
						gnutls_record_send($session, "$user $pw\n");
						$output = gnutls_record_recv($session, 10);
						$output = trim($output);

						if($output != "OK")
						{
							$errmsg = _('Login is incorrect');
							@socket_close($testsock);
						}

					}
					else
					{
						$errmsg = _('Daemon doesn\'t support TLS');				
					}
				}
				else
				{
					$errmsg = _('No PHP TLS support found');
				}		
			}
			else
			{
				socket_write($testsock, "PLAINTEXT\n");
				$output = socket_read ($testsock, 5);
				socket_write($testsock, "$user $pw\n");
				$output = socket_read ($testsock, 5);

				if($output != "OK\n")
				{
					$errmsg = _('Login is incorrect');
					@socket_close($testsock);
					unset($testsock);
				}
			}
		}
		else
		{
			$errmsg = _('IP address invalid');
		}
		
		@socket_close($testsock);
		return $errmsg;		  	
	}

	 /**
	  * establish a connection to the given node id
	  * 
	  * @param int $node
	  */
	  public function socket_connect($node)
	  {
	  	if(isset($this->sockets[$node]['socket']))
	  		return true;
	  	else
	  	{
	  		$this->sockets[$node] = array();
	  		$db =& OpenVCP::call('DB', 'getReference');
	  		$nodeinfo = $db->select($db->table_nodes, NULL, NULL , array('id' => $node));
			$this->sockets[$node]['socket'] = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
			if($nodeinfo[0]['ip'] != "")
			{

				socket_set_nonblock($this->sockets[$node]['socket'])
		     		or die("Unable to set nonblock on socket\n");

		  		$time = time();

				while (!@socket_connect($this->sockets[$node]['socket'], $nodeinfo[0]['ip'], $nodeinfo[0]['port']))
	  			{
	     				$err = socket_last_error($this->sockets[$node]['socket']);

		     			if ($err == 115 || $err == 114)
		     			{
		       				if ((time() - $time) >= $this->config['timeout'])
		       				{
		        				socket_close($this->sockets[$node]['socket']);
		        				return FALSE;
		       				}
		       		
						usleep(100);
		                    		continue;
	     				}

					return FALSE;
 		  		}

   				socket_set_block($this->sockets[$node]['socket']);

				if($nodeinfo[0]['tls'] == 'y' && function_exists('gnutls_session_new'))
                		{

					socket_write($this->sockets[$node]['socket'], "TLS\n");
					$output = socket_read ($this->sockets[$node]['socket'], 10);
					$output = trim($output);

					if($output == "OK")
					{
						$session = gnutls_session_new(GNUTLS_CLIENT);
						$cred = gnutls_credentials_new(CRED_ANON_CLIENT);
						gnutls_set_default_priority($session);
						gnutls_kx_set_priority($session, GNUTLS_KX_ANON_DH);
						gnutls_session_set_credentials($session, $cred);
						gnutls_session_set_socket($session, $this->sockets[$node]['socket']);
						gnutls_handshake($session);
						gnutls_record_send($session, "{$nodeinfo[0]['username']} {$nodeinfo[0]['password']}\n");
						$output = gnutls_record_recv($session, 10);
						$output = trim($output);
	
						if($output != "OK")
						{
							@socket_close($this->sockets[$node]['socket']);
							$this->sockets[$node] = FALSE;
						}
						else
						{
							$this->sockets[$node]['session'] = $session;
						}
					}

				}
				else
				{
					socket_write($this->sockets[$node]['socket'], "PLAINTEXT\n");
					$output = socket_read ($this->sockets[$node]['socket'], 5);
					socket_write($this->sockets[$node]['socket'], "{$nodeinfo[0]['username']} {$nodeinfo[0]['password']}\n");
					$output = socket_read ($this->sockets[$node]['socket'], 5);
	
					if($output != "OK\n")
					{
						@socket_close($this->sockets[$node]['socket']);
						unset($this->sockets[$node]);
					}
				}
			}
			else
			{
				unset($this->sockets[$node]);
			}
			
			return isset($this->sockets[$node]) ? true : false;
	  	}
	}
	
	/**
	 * Send's data to the connected node and 
	 * returns the answer of the node'
	 * 
	 * @param int $node
	 * @param string $args
	 * @return string $answer
	 */
	 public function socket_exec($node, $xmlObjectId)
	 {
		$XML =& OpenVCP::call('XML', 'getReference');
		
		if(!isset($this->sockets[$node]))
			$this->socket_connect($node);

		$statsocks = array($this->sockets[$node]['socket']);

		if(socket_select($read = NULL, $statsocks, $except = NULL, 0, 10000) > 0)
		{
			$args = $XML->getRequestObject($xmlObjectId);
			if($args[strlen($args)-1] != "\n")
				$args = $args."\n";

			if(isset($this->sockets[$node]['session']))
				gnutls_record_send($this->sockets[$node]['session'], $args);
			else
        		socket_write($this->sockets[$node]['socket'], $args);
		}
		else
		{
			return FALSE;
		}
		
		$statsocks = array($this->sockets[$node]['socket']);

		$output = "";
		
		
		if(($ret = socket_select($statsocks, $write = NULL, $except = NULL, 500, 100000)) != -1)
		{
			if($ret > 0)
			{
				if(isset($this->sockets[$node]['session']))
				{	
	     				
	     				do
					{
						$return = "";
		
						if(($ret = socket_select($statsocks, $write = NULL, $except = NULL, 0, 100000)) > 0)
						{
				
							$return = gnutls_record_recv($this->sockets[$node]['session'], 1000);
							$output .= $return;
						}
					}
					while($return != "");
	     				
	     				
	     				
	     				
				}					
				else
	     			{	
	     				socket_set_nonblock ($this->sockets[$node]['socket']);
	     				
	     				while(($return = socket_read($this->sockets[$node]['socket'], 1000, PHP_BINARY_READ)) != "")
	     				{
	     					$output .= $return;
	     					usleep(100);
	     				}
	     				
	     				socket_set_block ($this->sockets[$node]['socket']);
	     			}

				$output = trim($output);
				$xmlObjectId = $XML->newResponseObject($output);
			}
		}
		else
		{
			$xmlObjectId = 0;
		}
		
		return $xmlObjectId;
	}

}
?>
