<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */
class Vserver {

	private $vserverinfo = null;
	private $DB = null;
	private $XML = null;
	private $SOCKET = null;

    function __construct($vserver) {
    	$this->DB =& OpenVCP::call('DB', 'getReference');
    	if(is_numeric($vserver) && $vserver > 0)
    	{
    		$vserver = $this->DB->select($this->DB->table_vservers, NULL, NULL, array('id' => $vserver));
    	}
    	else if(is_string($vserver) && $vserver != "")
    	{
    		$vserver = $this->DB->select($this->DB->table_vservers, NULL, NULL, array('name' => $vserver));
    	}
    	else
    	{
    		OpenVCP::coreError("vserver does not exist");
    	}
    	$this->SOCKET =& OpenVCP::call('Socket', 'getReference');
		$this->XML	=& OpenVCP::call('XML', 'getReference');
   		$this->vserverinfo = $vserver[0];
    }
    
    /**
     * returns the adminid
     */
    public function getAdmin()
    {
    	return $this->vserverinfo['adminid'];
    }
    
    /**
     * returns the userid
     *
     * @return int
     */
    public function getUser()
    {
    	return $this->vserverinfo['userid'];
    }
    
    /**
     * returns the name of the vserver
     */
	public function getName()
	{
		return $this->vserverinfo['name'];
	}
	
	/**
	 * returns the hostname of the vserver
	 */
	public function getHostname()
	{
		$request = $this->XML->newRequestObject("vserver.get_hostname");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$values = $this->XML->getValues($response, "string");
		return $values[0];
	}
	
	/**
	 * returns the ip's of the vserver
	 * 
	 * @return XMLObject responseid
	 */
	public function getIps()
	{
		$request = $this->XML->newRequestObject("vserver.get_ips");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		return $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}
	
	/**
	 * returns the traffic of the
	 * given month and year
	 * 
	 * @param int $month
	 * @param int $year
	 */
	public function getTrafficTotal($month, $year)
	{
		$request = $this->XML->newRequestObject("vserver.traffic_total");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "int", $month);
		$this->XML->addValue($request, "int", $year);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return $this->XML->getNextStruct($response);
	}
	
	/**
	 * returns the current state
	 * 
	 * @return string state
	 */
	public function getState()
	{
		$request = $this->XML->newRequestObject("vserver.get_state");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$values = $this->XML->getValues($response, "string");
		return $values[0];
	}
	
	/**
	 * returns the current load
	 * 
	 * @param boolean $format
	 * @return string load
	 */
	public function getLoad($format = false)
	{
		$request = $this->XML->newRequestObject("vserver.get_load");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$values = $this->XML->getValues($response, "string");
		if($format)
		{
			$data = explode(" ", $values[0]);
			$returndata = "";
			for($a = 0; $a < 3; $a++)
			{
				if($data[$a] < 1) $returndata .= "<font color=\"green\">".$data[$a]."</font>"; 
				else if($data[$a] < 5) $returndata .= "<font color=\"#FFCC00\">".$data[$a]."</font>";
				else $returndata .= "<font color=\"red\">".$data[$a]."</font>";
				($a < 2 && $data[$a] != "") ? $returndata .= "|" : $returndata .= "";
			}
			return $returndata;
		}
		else
			return $values[0];
	}
	
	/**
	 * returns the crrent uptime
	 * 
	 * @param boolean $format
	 * @return string uptime
	 */
	public function getUptime($format)
	{
		$request = $this->XML->newRequestObject("node.get_uptime");
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$uptime = $this->XML->getValues($response, "string");
		$request = $this->XML->newRequestObject("vserver.get_uptime");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$biasuptime = $this->XML->getValues($response, "string");
		$biasuptime = $biasuptime[0];
		if($format)
		{
		 	$input = explode(" ", $uptime[0]);
		 	$biasuptime == "0 0" ? $input[0] = 0 : $biasuptime = $biasuptime;
		 	$input[0] = $input[0] - $biasuptime;
			if($input[0] != 0)
			{
			 	$reference = 3600 * 24;
			 	$days = 0;
			 	while($input[0] > ($reference * $days))
			 	{
			 		$days++;
			 	}
			 	$days--;
			 	$input[0] = $input[0] - ($days * $reference);
			 	$hours = 0;
			 	while($input[0] > (3600 * $hours))
			 	{
			 		$hours++;
			 	}
			 	$hours--;
			 	$input[0] = $input[0] - ($hours * 3600);
			 	$mins = 0;
			 	while($input[0] > ($mins * 60))
			 	{
			 		$mins++;
			 	}
			 	$mins = ($mins < 10) ? "0".$mins : $mins;
			 	$hours = ($hours < 10) ? "0".$hours : $hours;
			}
			else
			{
				$days = 0;
				$hours = "00";
				$mins = "00";
			}
		 	return $days." "._("days").", ".$hours.":".$mins;
		}
		else
			return $biasuptime;
	}
	
	/**
	 * returns the current filters set for the
	 * vserver
	 * 
	 * @return XMLObject
	 *
	 */
	public function getFilter()
	{
		$request = $this->XML->newRequestObject("vserver.get_filter");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		return $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}

	/**
	 * adds a new Rule to the vserver
	 *
	 * @param unknown_type $direction
	 * @param unknown_type $proto
	 * @param unknown_type $srcip
	 * @param unknown_type $srcport
	 * @param unknown_type $destip
	 * @param unknown_type $destport
	 * @param unknown_type $target
	 * @param unknown_type $match
	 * @param unknown_type $matchValue
	 */
	public function setFilter($direction, $proto, $srcip, $srcport, $destip, $destport, $target, $match, $matchValue)
	{
		$request = $this->XML->newRequestObject("vserver.set_filter");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addStructMember($request, "direction", "string", $direction);
		$this->XML->addStructMember($request, "proto", "string", $proto);
		$this->XML->addStructMember($request, "srcip", "string", $srcip);
		$this->XML->addStructMember($request, "srcport", "string", $srcport);
		$this->XML->addStructMember($request, "destip", "string", $destip);
		$this->XML->addStructMember($request, "destport", "string", $destport);
		$this->XML->addStructMember($request, "target", "string", $target);
		$this->XML->addStructMember($request, "match", "string", $match);
		$this->XML->addStructMember($request, "match-value", "string", $matchValue);
		$this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}
	
	/**
	 * Deletes a rule of the vserver
	 * specified by $id
	 *
	 * @param int $id
	 */
	public function delFilter($id)
	{
		$request = $this->XML->newRequestObject("vserver.del_filter");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$this->XML->addValue($request, "string", $id);
		$this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
	}
	
	/**
	 * starts a vserver
	 */
	public function controlStart()
	{
		$request = $this->XML->newRequestObject("vserver.start");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return;
	}

	/**
	 * stops a vserver
	 */
	public function controlStop()
	{
		$request = $this->XML->newRequestObject("vserver.stop");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		return;
	}
	
	/**
	 * removes a vserver
	 * 
	 * @return boolean success
	 */
	public function controlRemove()
	{
		$request = $this->XML->newRequestObject("vserver.remove");
		$this->XML->addValue($request, "string", $this->vserverinfo['name']);
		$response = $this->SOCKET->socket_exec($this->vserverinfo['nodeid'], $request);
		$result = $this->XML->getValues($response, "boolean");
		return $result[0];
	}

}
?>
