<?php
/**
 * OpenVCP GUI Core Extension
 * This extension is a built in one,
 * so there no need to load it anywhere.
 * It provides methods to create
 * the output of the interface.
 *
 * @author  Oliver Werner
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

final class OpenVCP_XML extends OpenVCP_Core {
	
	/**
	 * Core module version
	 *
	 * @var string
	 */
	protected $version = '0.3.0.1';
	
	/**
	 * Allow references
	 * 
	 * @var bool
	 */
	protected $getRef = true;
	
	/**
	 * contains the various xml objects
	 */
	private $objects;
	
	/**
	 * contains the element's for an xml
	 * object
	 */
	private $objectElements;
	
	/**
	 * creates a new request Object
	 * need's the method name to call
	 * 
	 * @param string $method
	 * @return int $objectId
	 */
	public function newRequestObject($method)
	{
		$objectId = count($this->objects);
		$this->objects[$objectId] = new DomDocument();
		$this->objectElements[$objectId]['root'] = $this->objects[$objectId]->createElement("methodCall");
		$methodElement = $this->objects[$objectId]->createElement("methodName");
		$methodObject = $this->objects[$objectId]->createTextNode($method);
		$methodElement->appendChild($methodObject);
		unset($methodObject);
		$this->objectElements[$objectId]['root']->appendChild($methodElement);
		unset($methodElement);
		return $objectId;
	}
	
	/**
	 * creates a new response Object
	 * @param string $response
	 * @return int $objectId
	 */
	public function newResponseObject($response)
	{
		$objectId = count($this->objects);
		$this->objects[$objectId] = new DomDocument();
		$this->objects[$objectId]->loadXML($response);
		$firstChild = $this->objects[$objectId]->firstChild;
		$params = $firstChild->firstChild;
		$param = $params->firstChild;
		while($param != NULL)
		{
			$value = $param->firstChild;
			$valueChild = $value->firstChild;
			switch($valueChild->nodeName)
			{
				case "string" : $this->parseValue($valueChild, "string", $objectId);
								break;
				case "int"	  : $this->parseValue($valueChild, "int", $objectId);
								break;
				case "double" : $this->parseValue($valueChild, "double", $objectId);
								break;
				case "boolean" : $this->parseValue($valueChild, "boolean", $objectId);
								break;
				case "struct" : $this->parseStruct($valueChild, $objectId);
								break;
				case "array"  : $this->parseArray($valueChild, $objectId);
								break;
			}
			$param = $param->nextSibling;
		}
		//var_dump($this->objectElements[$objectId]);
		return $objectId;
	}
	
	/**
	 * returns the created Request Object
	 * as a string for sending
	 * @return string xmlstring
	 */
	public function getRequestObject($objectId)
	{
		if($this->finishElements($objectId))
		{
			echo $this->objects[$objectId]->saveXML();
			return $this->objects[$objectId]->saveXML();
		}
		else
		{
			return null;
		}
	}
	
	//At this point the request functions will start
	
	/**
	 * finishs up a xml request
	 * @param int $objectId
	 */
	private function finishElements($objectId)
	{
		$elements = array();
		if(isset($this->objectElements[$objectId]['values']))
		{
			$elements[] = $this->finishValue($objectId);
		}
		if(isset($this->objectElements[$objectId]['array']))
		{
			$elements[] = $this->finishArray($objectId);
		}
		else if(isset($this->objectElements[$objectId]['struct']))
		{
			$elements[][] = $this->finishStruct($objectId);
		}
		$params = $this->objects[$objectId]->createElement("params");
		//var_dump($elements);
		if(is_array($elements))
		{
			foreach($elements as $element)
			{
				foreach($element as $key => $newElement)
				{
					//var_dump($newElement);
					$param = $this->objects[$objectId]->createElement("param");
					$param->appendChild($newElement);
					$params->appendChild($param);
				}
			}
			unset($elements);
		}
		$this->objectElements[$objectId]['root']->appendChild($params);
		unset($params);
		$this->objects[$objectId]->appendChild($this->objectElements[$objectId]['root']);
		unset($this->objectElements[$objectId]);
		return true;
	}
	
	/**
	 * creates a new array to the given request xml
	 * @param int $objectId
	 */
	private function createArray($objectId)
	{
		$this->objectElements[$objectId]['array'] = array();
	}
	
	/**
	 * finishs up a array
	 * @param int $objectId
	 */
	private function finishArray($objectId)
	{
		$arrayParam = $this->objects[$objectId]->createElement("array");
		$arrayData = $this->objects[$objectId]->createElement("data");
		foreach($this->objectElements[$objectId]['array'] as $data)
		{
			foreach($data as $type => $value)
			{
				$arrayValue = $this->objects[$objectId]->createElement("value");
				$arrayValueType = $this->objects[$objectId]->createElement($type);
				$arrayValueType->appendChild($this->objects[$objectId]->createTextNode($value));
				$arrayValue->appendChild($arrayValueType);
				$arrayData->appendChild($arrayValue);
			}
		}
		unset($this->objectElements[$objectId]['array']);
		$arrayParam->appendChild($arrayData);
		return $arrayParam;
	}
	
	/**
	 * adds a new struct to the request
	 * @param int $objectId
	 */
	private function createStruct($objectId)
	{
		$this->objectElements[$objectId]['struct'] = array();
	}
	
	/**
	 * finishs a struct
	 * @param int $objectId
	 */
	private function finishStruct($objectId)
	{
		$value = $this->objects[$objectId]->createElement("value");
		$struct = $this->objects[$objectId]->createElement("struct");
		//var_dump($this->objectElements[$objectId]['struct']);
		foreach($this->objectElements[$objectId]['struct'] as $member)
		{
			$structMember = $this->objects[$objectId]->createElement("member");
			$memberName = $this->objects[$objectId]->createElement("name");
			$memberName->appendChild($this->objects[$objectId]->createTextNode($member['name']));
			//var_dump($member['name']);
			$memberValue = $this->objects[$objectId]->createElement("value");
			//var_dump($member['type']);
			$memberValueType = $this->objects[$objectId]->createElement($member['type']);
			//var_dump($member['value']);
			$memberValueType->appendChild($this->objects[$objectId]->createTextNode($member['value']));
			$memberValue->appendChild($memberValueType);
			//unset($memberValueType);
			$structMember->appendChild($memberName);
			//unset($memberName);
			$structMember->appendChild($memberValue);
			//unset($memberValue);
			$struct->appendChild($structMember);
			//unset($structMember);
		}
		//unset($this->objectElements[$objectId]['struct']);
		$value->appendChild($struct);
		//unset($struct);
		return $value;		
	}
	
	/**
	 * creates a new value array
	 */
	private function createValue($objectId, $valueType)
	{
		if(!isset($this->objectElements[$objectId]['values']))
		{
			$this->objectElements[$objectId]['values'] = array();
		}
		if(!isset($this->objectElements[$objectId]['values'][$valueType]))
		{
			$this->objectElements[$objectId]['values'][$valueType] = array();
		}
	}
	
	/**
	 * finish up all values
	 * @param int $objectId
	 */
	private function finishValue($objectId)
	{
		foreach($this->objectElements[$objectId]['values'] as $type => $types)
		{
			foreach($types as $num => $value)
			{
				$valueElement = $this->objects[$objectId]->createElement("value");
				$valueType = $this->objects[$objectId]->createElement($type);
				$valueType->appendChild($this->objects[$objectId]->createTextNode($value));
				$valueElement->appendChild($valueType);
				$values[] = $valueElement;
			}
		}
		return $values;
	}
	
	/**
	 * This adds an Array Element to an existing array
	 * @param string $type
	 * @param string $value
	 */
	public function addArrayElement($objectId, $type, $value)
	{
		if(!isset($this->objectElements[$objectId]['array']))
		{
			$this->createArray($objectId);
		}
		$this->objectElements[$objectId]['array'][count($this->objectElements[$objectId]['array'])][$type] = $value;
	}

	/**
	 * adds a new Member to the request struct
	 * @param int $objectId
	 * @param string $name
	 * @param enum $type on of 'boolean','int','string','double','base64'
	 * @param string $value
	 */
	public function addStructMember($objectId, $name, $type, $value)
	{
		if(!isset($this->objectElements[$objectId]['struct']))
		{
			$this->createStruct($objectId);
		}
		$nextElement = count($this->objectElements[$objectId]['struct']);
		$this->objectElements[$objectId]['struct'][$nextElement]['name'] = $name;
		$this->objectElements[$objectId]['struct'][$nextElement]['type'] = $type;
		$this->objectElements[$objectId]['struct'][$nextElement]['value'] = $value;	
	}

	/**
	 * adds a new element of $type to the request
	 * @param int $objectId
	 * @param enum $type on of 'boolean','int','string','double','base64'
	 * @param string $value
	 */
	public function addValue($objectId, $type, $value)
	{
		$this->createValue($objectId, $type);
		$this->objectElements[$objectId]['values'][$type][] = $value;
	}
		
	//At this point the response functions will start
	
	/**
	 * parses values of $type into the response array
	 * @param DomNode $root
	 * @param string $type
	 * @param int $objectId
	 */
	private function parseValue($root, $type, $objectId)
	{
		$typeElement = $root->firstChild;
		$this->objectElements[$objectId][$type][] = $typeElement->nodeValue;
	}
	
	/**
	 * pareses the structs into the response array
	 * @param DomNode $root
	 * @param int $objectId
	 */
	private function parseStruct($root, $objectId)
	{
		$member = $root->firstChild;
		$structNum = count($this->objectElements[$objectId]["struct"]);
		$memberName = "";
		$this->objectElements[$objectId]["struct"]["counter"] = 0;
		while($member != NULL)
		{
			$childs = $member->firstChild;
			while($childs != NULL)
			{
				if($childs->nodeName == "name")
				{
					$memberName = $childs->nodeValue;
					$this->objectElements[$objectId]["struct"][$structNum][$memberName];
				}
				else if($childs->nodeName == "value")
				{
					$this->objectElements[$objectId]["struct"][$structNum][$memberName] = $childs->nodeValue;
				}
				$childs = $childs->nextSibling;
			}
			$member = $member->nextSibling;
		}
	}

	/**
	 * pareses the arrays into the response array
	 * @param DomNode $root
	 * @param int $objectId
	 */	
	private function parseArray($root, $objectId)
	{
		$this->objectElements[$objectId]["array"]["counter"] = 0;
		$data = $root->firstChild;
		$value = $data->firstChild;
		$arrayNum = count($this->objectElements[$objectId]["array"]);
		while($value != NULL)
		{
			$this->objectElements[$objectId]["array"][$arrayNum][] = $value->nodeValue;
			$value = $value->nextSibling;
		}
	}
	
	/**
	 * returns the elements of the array as an array
	 * @param int $objectId
	 * @return array array elements
	 */
	public function getNextArray($objectId)
	{
		if($this->objectElements[$objectId]["array"]["counter"] < count($this->objectElements[$objectId]["array"]))
		{
			while(!is_array($this->objectElements[$objectId]["array"][$this->objectElements[$objectId]["array"]["counter"]]) && $this->objectElements[$objectId]["array"]["counter"] < count($this->objectElements[$objectId]["array"]))
			{
				$this->objectElements[$objectId]["array"]["counter"]++;
			}
				
			$this->objectElements[$objectId]["array"]["counter"]++;
			return $this->objectElements[$objectId]["array"][$this->objectElements[$objectId]["array"]["counter"]-1];
		}
		else
		{
			return NULL;
		}		
	}
	
	/**
	 * returns a struct or when last struct is reached
	 * null
	 * @param int $objectId
	 * @return array struct or null
	 */
	public function getNextStruct($objectId)
	{
		if($this->objectElements[$objectId]["struct"]["counter"] < count($this->objectElements[$objectId]["struct"]))
		{
			while(!is_array($this->objectElements[$objectId]["struct"][$this->objectElements[$objectId]["struct"]["counter"]]) && $this->objectElements[$objectId]["struct"]["counter"] < count($this->objectElements[$objectId]["struct"]))
			{
				$this->objectElements[$objectId]["struct"]["counter"]++;
			}
				
			$this->objectElements[$objectId]["struct"]["counter"]++;
			return $this->objectElements[$objectId]["struct"][$this->objectElements[$objectId]["struct"]["counter"]-1];
		}
		else
		{
			return NULL;
		}
	}
	
	/**
	 * gets the int values of a response and
	 * returns an array with the variuos values
	 * @param int $objectId
	 * @param enum $type on of 'boolean','int','string','double','base64'
	 * @return array array of the values
	 */
	public function getValues($objectId, $type)
	{
		if(is_array($this->objectElements[$objectId][$type]))
			return $this->objectElements[$objectId][$type];
		else
			return NULL;
	}
	
}
?>
