<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */
 
/**
 * Creates contents for a dropdown menu
 * with the given mysql table
 * 
 * @param string $table
 * @param array $field
 * @param array $compare
 * @param string $firstline
 * 
 * @return array $contents
 */
function createDropdownContent($table, array $field = NULL, array $compare = NULL, $firstline = "", $order = null)
{
	global $DB;
	$result = $DB->select($DB->$table, NULL, $field, $compare, $order);
	$contents = array();
	$firstline != "" ? $contents[$firstline] = $firstline : "";
	foreach($result as $res)
	{
		$contents[$res[$field[1]]] = $res[$field[0]];
	}
	return $contents;
}

/**
 * checks emailadress whether is a valid emailadress or not
 * @param string $email
 * 
 * @return bool $valid
 */
function checkEmail($email)
{
	return preg_match("/(.*)@[a-z|A-Z|0-9|á|é|í|ó|ú|à|è|ì|ò|ù|ä|ë|ï|ö|ü|â|ê|î|ô|û|ñ|ç|þ|æ|ð|å|ý|-]+\.[a-z|A-Z]+/i", $email) ? true : false;
}

/**
 * creates a password
 * 
 * @param int $length
 * 
 * @return array $pw
 */
function createPassword($length=8)
{
   mt_srand((double)microtime()*1000000);
   $newstring="";

   if($length>0)
   {

      while(strlen($newstring)<$length)
	{
           switch(mt_rand(1,3))
		{
               case 1: $newstring.=chr(mt_rand(48,57)); break;  // 0-9
               case 2: $newstring.=chr(mt_rand(65,90)); break;  // A-Z
               case 3: $newstring.=chr(mt_rand(97,122)); break; // a-z
           }
       }
   }
   return $newstring;
}

/**
 * returns the requested setting
 * 
 * @param string $key
 * @return $value
 * @author Oliver Werner <mail@cryptronic.de>
 */
function getSetting($key)
{
	global $DB, $USERINFO;
	$settings = $DB->select($DB->table_admin_settings, NULL, NULL, array('adminid' => $USERINFO['id'],'AND', 'key' => $key));
	if(is_array($settings[0]) && $settings[0]['value'] != "")
	{
		return $settings[0]['value'];
	}
	else
	{
		$settings = $DB->select($DB->table_settings, NULL, NULL, array('key' => $key));
		return (is_array($settings) && $settings[0]['value'] != "") ? $settings[0]['value'] : false;
	}
}

/**
 * updates the settings
 * 
 * @param string $key
 * @param string $value
 * @return int $return
 * @author Oliver Werner <mail@cryptronic.de> 
 */
function updateSetting($key, $value)
{
	global $DB, $USERINFO;
	$setting = $DB->select($DB->table_admin_settings, NULL, NULL, array('adminid' => $USERINFO['id'], 'AND', 'key' => $key));
	if(is_array($setting) && $setting[0]['key'] == $key)
	{
		$DB->update($DB->table_admin_settings, array('value' => $value), array('adminid' => $USERINFO['id'], 'AND', 'key' => $key));
	}
	else
		$DB->update($DB->table_settings, array('value' => $value), array('key' => $key));		
	return 1;
}

?>
